import smtplib

TEMPLATE = '''
From: {from_addr}
To: {to_addrs}
Subject: {subject}

{message}
'''

def send_mail(smtp_server, from_addr, to_addrs, subject, message):
    with smtplib.SMTP(smtp_server) as smtp:
        smtp.sendmail(from_addr, to_addrs, TEMPLATE.format(
            from_addr=from_addr,
            to_addrs=to_addrs,
            subject=subject,
            message=message
        ))



#!/usr/bin/env python3

"""Create an empty archive in the RADAR service to reserve a DOI and a RADAR ID.

Description
-----------

This script creates an empty archive in the RADAR service in order to reserve a DOI and a RADAR ID.
Both are stored in the CodeMeta metadata file provided as input and can be later used by the script ``create_radar.py``
to populate the RADAR archive.

Usage
-----

.. argparse::
    :module: facile_rs.prepare_radar
    :func: create_parser
    :prog: prepare_radar.py

"""
from pathlib import Path

from rich import print

from .utils import cli
from .utils.exceptions import ParserError
from .utils.http import is_remote
from .utils.metadata import CodemetaMetadata, RadarMetadata
from .utils.radar import create_radar_dataset, fetch_radar_token, prepare_radar_dataset


def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('--codemeta-location', dest='CODEMETA_LOCATION',
                        help='Location of the main codemeta.json JSON file')
    parser.add_argument('--radar-url', dest='RADAR_URL', required=True,
                        help='URL of the RADAR service.')
    parser.add_argument('--radar-username', dest='RADAR_USERNAME', required=True,
                        help='Username for the RADAR service.')
    parser.add_argument('--radar-password', dest='RADAR_PASSWORD', required=True,
                        help='Password for the RADAR service.')
    parser.add_argument('--radar-client-id', dest='RADAR_CLIENT_ID', required=True,
                        help='Client ID for the RADAR service.')
    parser.add_argument('--radar-client-secret', dest='RADAR_CLIENT_SECRET', required=True,
                        help='Client secret for the RADAR service.')
    parser.add_argument('--radar-workspace-id', dest='RADAR_WORKSPACE_ID', required=True,
                        help='Workspace ID for the RADAR service.')
    parser.add_argument('--radar-redirect-url', dest='RADAR_REDIRECT_URL', required=True,
                        help='Redirect URL for the OAuth workflow of the RADAR service.')
    parser.add_argument('--radar-email', dest='RADAR_EMAIL', required=True,
                        help='Email for the RADAR metadata.')
    parser.add_argument('--radar-backlink', dest='RADAR_BACKLINK', required=True,
                        help='Backlink for the RADAR metadata.')
    parser.add_argument('--keep-previous-doi', action='store_true', dest='KEEP_PREVIOUS_DOI',
                        help='When creating a new version, keep the previous DOI in the Codemeta file, '
                        'in the field "isBasedOn". '
                        'By default, the previous DOI is removed.')
    parser.add_argument('--dry', action='store_true', dest='DRY',
                        help='Perform a dry run, do not upload anything.')
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')
    return parser


def main(args):
    if args.CODEMETA_LOCATION:
        if is_remote(args.CODEMETA_LOCATION):
            raise ParserError(f'Location {args.CODEMETA_LOCATION} is not a local file.')

        codemeta = CodemetaMetadata()
        codemeta.fetch(args.CODEMETA_LOCATION)

        name = '{name} ({version}, in preparation)'.format(**codemeta.data)
    else:
        name = 'in preparation'

    radar_metadata = RadarMetadata({'name': name}, args.RADAR_EMAIL, args.RADAR_BACKLINK)
    radar_dict = radar_metadata.as_dict()

    if not args.DRY:
        # obtain oauth token
        headers = fetch_radar_token(args.RADAR_URL, args.RADAR_CLIENT_ID, args.RADAR_CLIENT_SECRET,
                                    args.RADAR_REDIRECT_URL, args.RADAR_USERNAME, args.RADAR_PASSWORD)

        # create radar dataset
        dataset_id = create_radar_dataset(args.RADAR_URL, args.RADAR_WORKSPACE_ID, headers, radar_dict)
        dataset = prepare_radar_dataset(args.RADAR_URL, dataset_id, headers)

        doi = dataset.get('descriptiveMetadata', {}).get('identifier', {}).get('value')

        if args.CODEMETA_LOCATION:
            codemeta.update_identifier(new_doi=doi, new_archive_id=dataset_id, archive_type='RADAR',
                                       keep_previous_doi=args.KEEP_PREVIOUS_DOI)

            Path(args.CODEMETA_LOCATION).expanduser().write_text(codemeta.to_json())
        else:
            print(dataset)
    else:
        print(radar_dict)


def main_deprecated():
    cli.main_deprecated(__name__)


if __name__ == "__main__":
    main_deprecated()



import ast
import json
import sys
from os import path

from facile_rs.utils.cli import main

SCRIPT_DIR = path.dirname(path.realpath(__file__))
METADATA_DIR = path.join(path.dirname(SCRIPT_DIR), 'utils', 'metadata', 'tests')

CODEMETA_LOCATION = path.join(METADATA_DIR, 'codemeta_test.json')
RADAR_URL = 'https://radar.kit.edu'
RADAR_USERNAME = 'testuser'
RADAR_PASSWORD = 'testpassword'
RADAR_CLIENT_ID = 'testclientid'
RADAR_CLIENT_SECRET = 'testclientsecret'
RADAR_WORKSPACE_ID = 'testworkspaceid'
RADAR_REDIRECT_URL = 'https://example.com'
RADAR_EMAIL = 'testuser@example.com'
RADAR_BACKLINK = 'https://example.com'


def test_dry_nocodemeta_cli(monkeypatch, capsys):
    """
    Test the function runs with CLI options and that the radar metadata is printed in
    the standard output in dry mode.
    """
    monkeypatch.delenv('CODEMETA_LOCATION', raising=False)
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'radar',
                            'prepare',
                            '--radar-url', RADAR_URL,
                            '--radar-username', RADAR_USERNAME,
                            '--radar-password', RADAR_PASSWORD,
                            '--radar-client-id', RADAR_CLIENT_ID,
                            '--radar-client-secret', RADAR_CLIENT_SECRET,
                            '--radar-workspace-id', RADAR_WORKSPACE_ID,
                            '--radar-redirect-url', RADAR_REDIRECT_URL,
                            '--radar-email', RADAR_EMAIL,
                            '--radar-backlink', RADAR_BACKLINK,
                            '--dry'
                        ])
    main()
    captured = ast.literal_eval(capsys.readouterr().out)
    assert 'technicalMetadata' in captured
    assert 'descriptiveMetadata' in captured
    assert captured['descriptiveMetadata'].get('title', '') == 'in preparation'


def test_dry_env(monkeypatch, capsys):
    """
    Test the function runs with options set as environment variables and
    that the CodeMeta metadata is handled correctly.
    """
    monkeypatch.setenv('CODEMETA_LOCATION', CODEMETA_LOCATION)
    monkeypatch.setenv('RADAR_URL', RADAR_URL)
    monkeypatch.setenv('RADAR_USERNAME', RADAR_USERNAME)
    monkeypatch.setenv('RADAR_PASSWORD', RADAR_PASSWORD)
    monkeypatch.setenv('RADAR_CLIENT_ID', RADAR_CLIENT_ID)
    monkeypatch.setenv('RADAR_CLIENT_SECRET', RADAR_CLIENT_SECRET)
    monkeypatch.setenv('RADAR_WORKSPACE_ID', RADAR_WORKSPACE_ID)
    monkeypatch.setenv('RADAR_REDIRECT_URL', RADAR_REDIRECT_URL)
    monkeypatch.setenv('RADAR_EMAIL', RADAR_EMAIL)
    monkeypatch.setenv('RADAR_BACKLINK', RADAR_BACKLINK)
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'radar',
                            'prepare',
                            '--dry'
                        ])
    main()
    captured = ast.literal_eval(capsys.readouterr().out)
    assert 'technicalMetadata' in captured
    assert 'descriptiveMetadata' in captured

    # Test if the title is correctly set to the name and version of the codemeta file
    with open(CODEMETA_LOCATION) as f:
        codemeta = json.load(f)
    codemeta_name = codemeta.get('name')
    codemeta_version = codemeta.get('version')
    assert captured['descriptiveMetadata'].get('title', '') == f'{codemeta_name} ({codemeta_version}, in preparation)'



import sys
from os import path

import pytest

from facile_rs.utils.cli import main

SCRIPT_DIR = path.dirname(path.realpath(__file__))
METADATA_DIR = path.join(path.dirname(SCRIPT_DIR), 'utils', 'metadata', 'tests')

CODEMETA_BASENAME = 'codemeta_test.json'
CODEMETA_LOCATION = path.join(METADATA_DIR, CODEMETA_BASENAME)
CREATORS_LOCATIONS = path.join(METADATA_DIR, 'codemeta_authors_test.json')
CONTRIBUTORS_LOCATIONS = path.join(METADATA_DIR, 'codemeta_contributors_test.json')
ASSETS = [path.join(METADATA_DIR, 'codemeta_test.json'), '']


def test_error_zenodo_asset_exists(monkeypatch, tmp_path, capsys):
    """
    Error is raised when an object with the same name as an asset already exists in the Zenodo path.
    """
    zenodo_path = tmp_path / 'zenodo_path'
    zenodo_path.mkdir()
    # Create a dummy file in the zenodo path to simulate an existing asset
    (zenodo_path / CODEMETA_BASENAME).touch()
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'zenodo',
                            'upload',
                            '--codemeta-location', CODEMETA_LOCATION,
                            '--zenodo-path', str(zenodo_path),
                            '--zenodo-url', 'https://sandbox.zenodo.org',
                            '--zenodo-token', '1234567890',
                            '--dry',
                            *ASSETS
                        ])
    with pytest.raises(SystemExit, match='^2$'):
        main()
    captured = capsys.readouterr().err
    assert 'already exists' in captured


def test_cli_dry(monkeypatch, tmp_path):
    """
    Test that the function runs with required CLI options in dry mode.
    """
    zenodo_path = tmp_path / 'zenodo_path'
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'zenodo',
                            'upload',
                            '--codemeta-location', CODEMETA_LOCATION,
                            '--zenodo-path', str(zenodo_path),
                            '--zenodo-url', 'https://sandbox.zenodo.org',
                            '--zenodo-token', '1234567890',
                            '--dry'
                        ])
    main()



import logging
from urllib.parse import urlparse

import requests

logger = logging.getLogger(__file__)


def fetch_radar_token(radar_url, client_id, client_secret, redirect_url, username, password):
    """
    Fetch RADAR token using the RADAR API.

    :param radar_url: URL to the RADAR repository
    :param client_id: RADAR client ID
    :param client_secret: RADAR client secret
    :param redirect_url: RADAR redirect URL
    :param username: RADAR username
    :param password: RADAR password
    :return: RADAR token for the given user
    """
    url = radar_url + '/radar/api/tokens'
    try:
        response = requests.post(url, json={
            'clientId': client_id,
            'clientSecret': client_secret,
            'redirectUrl': redirect_url,
            'userName': username,
            'userPassword': password
        })
        response.raise_for_status()
        logger.debug('response = %s', response.json())
    except requests.exceptions.HTTPError as e:
        logger.error(response.text)
        raise e

    tokens = response.json()
    return {
        'Authorization': 'Bearer {}'.format(tokens['access_token'])
    }


def create_radar_dataset(radar_url, workspace_id, headers, radar_dict):
    """
    Create a dataset in the given RADAR workspace, using the token provided in headers.

    :param radar_url: URL to the RADAR repository
    :param workspace_id: RADAR workspace ID
    :param headers: request headers. Typically the RADAR token, as returned by fetch_radar_token
    :param radar_dict: RADAR metadata dictionary, as returned by RadarMetadata.as_dict()
    :return: RADAR dataset ID
    """
    url = radar_url + f'/radar/api/workspaces/{workspace_id}/datasets'
    try:
        response = requests.post(url, headers=headers, json=radar_dict)
        response.raise_for_status()
        logger.debug('response = %s', response.json())
        return response.json()['id']
    except requests.exceptions.HTTPError as e:
        logger.error(response.text)
        raise e


def prepare_radar_dataset(radar_url, dataset_id, headers):
    """"
    Prepare a dataset for review in RADAR.

    :param radar_url: URL to the RADAR repository
    :param dataset_id: RADAR dataset ID
    :param headers: request headers. Typically the RADAR token, as returned by fetch_radar_token
    :return: RADAR response to the request
    """

    review_url = radar_url + f'/radar/api/datasets/{dataset_id}/startreview'
    try:
        response = requests.post(review_url, headers=headers)
        if response.status_code == 422:
            dataset_url = radar_url + f'/radar/api/datasets/{dataset_id}/'
            response = requests.get(dataset_url, headers=headers)
            response.raise_for_status()
            logger.debug('response = %s', response.json())
            return response.json()
        else:
            logger.debug('response = %s', response.json())
            raise RuntimeError('startreview did not return 422')
    except requests.exceptions.HTTPError as e:
        logger.error(response.text)
        raise e


def update_radar_dataset(radar_url, dataset_id, headers, radar_dict):
    """
    Update a dataset's metadata in the given RADAR workspace.

    :param radar_url: URL to the RADAR repository
    :param dataset_id: RADAR dataset ID
    :param headers: request headers. Typically the RADAR token, as returned by fetch_radar_token
    :param radar_dict: RADAR metadata dictionary, as returned by RadarMetadata.as_dict()
    :return: RADAR dataset ID
    """
    url = radar_url + f'/radar/api/datasets/{dataset_id}'
    try:
        response = requests.put(url, headers=headers, json=radar_dict)
        response.raise_for_status()
        logger.debug('response = %s', response.json())
        return response.json()['id']
    except requests.exceptions.HTTPError as e:
        logger.error(response.text)
        raise e


def upload_radar_assets(radar_url, dataset_id, headers, assets, path):
    """
    Upload assets to a RADAR dataset.

    :param radar_url: URL to the RADAR repository
    :param dataset_id: RADAR dataset ID
    :param headers: request headers. Typically the RADAR token, as returned by fetch_radar_token
    :param assets: locations of assets to upload
    :type assets: list
    :param path: location where the assets are collected before upload
    """
    url = radar_url + f'/radar-ingest/upload/{dataset_id}/file'
    for location in assets:
        parsed_url = urlparse(location)
        target = path / parsed_url.path.split('/')[-1]
        files = {'upload_file': open(target, 'rb')}

        try:
            response = requests.post(url, files=files, headers=headers)
            response.raise_for_status()
            logger.debug('response = %s', response.text)
        except requests.exceptions.HTTPError as e:
            logger.error(response.text)
            raise e



import json
import logging

import requests

logger = logging.getLogger(__file__)


class ZenodoMetadata:

    prefixes = {
        'doi': 'https://doi.org/',
        'orcid': 'https://orcid.org/',
        'ror': 'https://ror.org/'
    }

    ZENODO_CONTRIBUTORTYPES = [
        "contactperson",
        "datacollector",
        "datacurator",
        "datamanager",
        "distributor",
        "editor",
        "hostinginstitution",
        "other",
        "producer",
        "projectleader",
        "projectmanager",
        "projectmember",
        "registrationagency",
        "registrationauthority",
        "relatedperson",
        "researcher",
        "researchgroup",
        "rightsholder",
        "sponsor",
        "supervisor",
        "workpackageleader"
    ]

    ZENODO_RELATIONSHIPS = [
        "iscitedby",
        "cites",
        "issupplementto",
        "issupplementedby",
        "iscontinuedby",
        "continues",
        "isdescribedby",
        "describes",
        "hasmetadata",
        "ismetadatafor",
        "isnewversionof",
        "ispreviousversionof",
        "ispartof",
        "haspart",
        "isreferencedby",
        "references",
        "isdocumentedby",
        "documents",
        "iscompiledby",
        "compiles",
        "isvariantformof",
        "isoriginalformof",
        "isidenticalto",
        "isalternateidentifier",
        "isreviewedby",
        "reviews",
        "isderivedfrom",
        "issourceof",
        "requires",
        "isrequiredby",
        "isobsoletedby",
        "obsoletes"
    ]

    def __init__(self, data={}):
        """
        Initialize the ZenodoMetadata object from CodeMeta metadata.

        :param data: CodeMeta metadata, typically the data attribute of a CodemetaMetadata instance.
        :type data: dict
        """
        self.data = data
        logger.debug('data = %s', self.data)

    def get_license_id_from_spdx(self, spdx_id):
        """
        Get Zenodo license ID from SPDX identifier. Return None if the id could not be validated.

        :param spdx_id: SPDX license identifier
        :return: Zenodo identifier for the license or None if not found
        """
        # Default mapping
        zenodo_id = spdx_id.lower()

        # Handle exceptions
        if zenodo_id == "lgpl-3.0":
            return "lgpl-3.0-only"

        # Validate license id, use 'notspecified' if license cannot be validated
        r = requests.get(f"https://zenodo.org/api/vocabularies/licenses/{zenodo_id}")
        if r.status_code != 200:
            logger.info(f'Zenodo license ID {zenodo_id} could not be validated...')
            zenodo_id = None

        return zenodo_id

    def to_rights(self, license):
        """
        Convert a CodeMeta license to a Zenodo rights object.

        :param license: CodeMeta license, URL or Schema.org CreativeWork object
        :type license: str or dict
        :return: Zenodo rights object
        :rtype: dict
        """
        zenodo_right = {}
        # Attempt to find license SPDX identifier
        licenseName = "unknown"
        if isinstance(license, str):
            licenseName = license
        elif isinstance(license, dict):
            if 'url' in license:
                licenseName = license['url']
            elif 'name' in license:
                licenseName = license['name']
        if licenseName.startswith("https://spdx.org/licenses/"):
            licenseName = licenseName.replace("https://spdx.org/licenses/", "")
        zenodo_id = self.get_license_id_from_spdx(licenseName)

        if zenodo_id:
            zenodo_right['id'] = zenodo_id
        else:
            if 'name' in license:
                zenodo_right['title'] = {'en': license['name']}
            if 'url' in license:
                zenodo_right['link'] = license['url']
            if 'description' in license:
                zenodo_right['description'] = {'en': license['description']}
        return zenodo_right

    def to_person_or_org(self, value, contributorType=None):
        """
        Converts a Person or Organization object to a Zenodo "person_or_org".
        If contributorType is not None, it will be converted to a Zenodo "contributor_person_or_org".

        :param value: CodeMeta Person or Organization object
        :type value: dict
        :param contributorType: None or type of contributor, taken from the Zenodo controlled vocabulary
            defined in self.ZENODO_CONTRIBUTORTYPES
        :type contributorType: None or str
        :return: Zenodo person_or_org or contributor_person_or_org object
        :rtype: dict
        """
        # Provide default value
        out_entry = {}

        if contributorType:
            if contributorType in self.ZENODO_CONTRIBUTORTYPES:
                out_entry['role'] = {
                    'id': contributorType
                }
            else:
                logger.info(f'Contributor type {contributorType} is not in the Zenodo controlled vocabulary,'\
                    'replacing with \'other\'...')
                out_entry['role'] = {
                    'id': 'other'
                }

        # Define is entry is a person or an organization
        person_or_org = {}
        if 'type' in value and value['type'] == 'Organization':
            person_or_org['type'] = 'organizational'
            person_or_org['name'] = value.get('name', '')
        elif ('type' in value and value['type'] == 'Person') or 'familyName' in value:
            person_or_org['type'] = 'personal'
            person_or_org['family_name'] = value.get('familyName', '')
            person_or_org['given_name'] = value.get('givenName', '')
        else:
            person_or_org['type'] = 'organizational'
            person_or_org['name'] = value.get('name', '')

        for id in ['id', '@id']:
            if id in value:
                person_or_org['identifiers'] = []
                for scheme, prefix in self.prefixes.items():
                    if value[id].startswith(prefix):
                        person_or_org['identifiers'].append(
                            {
                                'scheme': scheme,
                                'identifier': value[id].replace(prefix, '')
                            }
                        )
        out_entry['person_or_org'] = person_or_org

        # Add affiliations if type is person
        if person_or_org['type'] == 'personal':
            if isinstance(value.get('affiliation', {}), dict):
                value['affiliation'] = [value.get('affiliation', {})]
            for affiliation in value.get('affiliation', []):
                if 'affiliations' not in out_entry:
                    out_entry['affiliations'] = []
                id_found = False
                for id in ['id', '@id']:
                    if id in affiliation:
                        if affiliation[id].startswith(self.prefixes['ror']):
                            out_entry['affiliations'].append({
                                                            'id': affiliation[id].replace(self.prefixes['ror'], ''),
                                                            'name': affiliation.get('name', '')})
                            id_found = True
                # If ROR not found, add affiliation name only
                if not id_found:
                    out_entry['affiliations'].append({'name': affiliation.get('name', '')})
        return out_entry

    def person_or_org_to_string(self, zenodo_person_or_org):
        """
        Convert a Zenodo person_or_org object to a string containing its name.
        """
        res = zenodo_person_or_org.get('name', '')
        if zenodo_person_or_org.get('type') == 'personal':
            res = f"{zenodo_person_or_org.get('given_name', '')} {zenodo_person_or_org.get('family_name', '')}"
        return res

    def validate_funding_identifier(self, funding_identifier):
        """
        Validate a funding identifier against the Zenodo API. If funding could be validated,
        return the funding object, otherwise return an empty dictionary.

        :param funding_identifier: funding identifier to validate
        :type funding_identifier: str
        :return: Zenodo funding object or empty dictionary
        :rtype: dict
        """
        r = requests.get('https://zenodo.org/api/awards',
            params={'q': f'number:{funding_identifier}'}
            )
        r_json = r.json()
        if r.status_code == 200 and r_json['hits']['total'] == 1:
            return r_json['hits']['hits'][0]
        logger.info(f'Funding identifier {funding_identifier} could not be validated...')
        return {}

    def to_funding(self, funding):
        """
        Convert a CodeMeta funding object to a Zenodo funding object.

        :param funding: CodeMeta funding object
        :type funding: dict or str
        :return: Zenodo funding object
        """
        # Case when funding is a string
        if isinstance(funding, str):
            zenodo_funding = {
                'award': {'title': {"en": funding}},
                'funder': {'name': 'Unknown funder'}
            }
            # Case when funder is provided as a single element (typically with CodeMeta generator)
            if 'funder' in self.data and not isinstance(self.data['funder'], list):
                zenodo_funding['funder'] = self.to_funder(self.data['funder'])
            return zenodo_funding

        # Case when funding is a dictionary
        zenodo_funding = {}
        # Attempt to validate funding identifier
        if 'identifier' in funding:
            validated_funding = self.validate_funding_identifier(funding['identifier'])
            if validated_funding:
                zenodo_funding['award'] = {
                    'id': validated_funding['id']
                }
                zenodo_funding['funder'] = {
                    'id': validated_funding['funder']['id']
                }
        # Fallback to custom funding object
        if not zenodo_funding:
            if 'funder' in funding:
                zenodo_funding['funder'] = self.to_funder(funding['funder'])
            else:
                zenodo_funding['funder'] = {'name': 'Unknown funder'}

            zenodo_funding['award'] = {'title': {"en": funding.get('name', '')}}
            if 'identifier' in funding:
                zenodo_funding['award']['number'] = funding['identifier']
            if 'url' in funding:
                zenodo_funding['award']['identifiers'] = [{
                        'scheme': 'url',
                        'identifier': funding['url']
                    }]
        return zenodo_funding

    def validate_funder_identifier(self, funder_identifier):
        """
        Validate a funder identifier against the Zenodo API. If funder could be validated,
        return the funder identifier, otherwise return None.
        Supports plain identifier or full ROR URL.
        """
        funder_identifier = funder_identifier.replace(self.prefixes['ror'], '')
        r = requests.get('https://zenodo.org/api/funders',
            params={'q': f'id:{funder_identifier}'}
            )
        r_json = r.json()
        if r.status_code == 200 and r_json['hits']['total'] == 1:
            return r_json['hits']['hits'][0]['id']
        logger.info(f'Funder identifier {funder_identifier} could not be validated...')
        return None

    def to_funder(self, funder):
        """
        Convert a CodeMeta funder to a Zenodo funder object.
        """
        zenodo_funder = {}
        if isinstance(funder, str):
            funderid = self.validate_funder_identifier(funder)
        elif isinstance(funder, dict) and 'id' in funder:
            funderid = self.validate_funder_identifier(funder['id'])
        elif isinstance(funder, dict) and '@id' in funder:
            funderid = self.validate_funder_identifier(funder['@id'])
        else:
            funderid = None
        if funderid:
            zenodo_funder['id'] = funderid
        else:
            if isinstance(funder, str):
                zenodo_funder['name'] = funder
            else:
                if 'name' in funder:
                    zenodo_funder['name'] = funder['name']
                elif 'url' in funder:
                    zenodo_funder['name'] = funder['url']
        return zenodo_funder

    def to_related_identifier(self, identifier, relation):
        """
        Convert an identifier to a Zenodo related identifier object.

        :param identifier: Persistent identifier of related publication or dataset.
            Supported identifier schemes:
            ARK, arXiv, Bibcode, DOI, EAN13, EISSN, Handle, IGSN, ISBN, ISSN, ISTC, LISSN, LSID,
            PubMed ID, PURL, UPC, URL, URN, W3ID.
        :type identifier: str
        :param relation: relationship described with controlled vocabulary defined in self.ZENODO_RELATIONSHIPS
        :type relation: str
        :return: Zenodo related identifier object
        :rtype: dict or None
        """
        if relation in self.ZENODO_RELATIONSHIPS:
            zenodo_related_identifier = {
                'relation_type': {'id': relation},
                'identifier': identifier
            }
        else:
            logger.info(f'Relation {relation} is not in the Zenodo controlled vocabulary, skipping...')
            zenodo_related_identifier = None
        return zenodo_related_identifier

    def to_subjects(self, keywords):
        """
        Convert CodeMeta keywords to Zenodo subjects.
        """
        zenodo_subjects = []
        for keyword in keywords:
            if isinstance(keyword, str):
                zenodo_subjects.append({'subject': keyword})
            else:
                if 'name' in keyword:
                    if keyword.get('@type') == 'DefinedTerm' and 'url' in keyword:
                        zenodo_subjects.append({
                            # Should be:
                            # 'id': keyword['url']
                            # but caused an error 500 in Zenodo, to be investigated...
                            'subject': keyword['url']
                        })
                    else:
                        zenodo_subjects.append({'subject': keyword['name']})
        return zenodo_subjects

    def add_to_array_field(self, zenodo_dict, key, value):
        """
        Add an object to a field of type "array" in a Zenodo metadata dictionary.
        Add the field if it doesn't exist yet, append the value to the field otherwise.
        Does nothing if value is None.

        :param zenodo_dict: Zenodo metadata dictionary; will be modified in place
        :param key: key to be added or updated
        :param value: value to be added. If None, the function does nothing.
        """
        if value:
            if key not in zenodo_dict['metadata']:
                zenodo_dict['metadata'][key] = []
            zenodo_dict['metadata'][key].append(value)

    def as_dict(self):
        """
        Prepare Zenodo payload to be passed to the Zenodo API.

        :return: Zenodo metadata dictionary
        """

        zenodo_dict = {
            'metadata': {
                'language': 'eng'
            }
        }

        if 'name' in self.data:
            zenodo_dict['metadata']['title'] = self.data['name']

        if 'alternateName' in self.data:
            self.add_to_array_field(zenodo_dict,
                'additional_titles',
                {
                    'title': self.data['alternateName'],
                    'type': {'id': 'alternative-title'}
                })

        if 'version' in self.data:
            zenodo_dict['metadata']['version'] = self.data['version']

        if 'dateModified' in self.data:
            zenodo_dict['metadata']['publication_date'] = self.data['dateModified']

        if 'sameAs' in self.data:
            related_id = self.to_related_identifier(self.data['sameAs'], 'isidenticalto')
            self.add_to_array_field(zenodo_dict, 'related_identifiers', related_id)

        if 'downloadUrl' in self.data:
            related_id = self.to_related_identifier(self.data['downloadUrl'], 'issourceof')
            self.add_to_array_field(zenodo_dict, 'related_identifiers', related_id)

        if 'referencePublication' in self.data \
            and self.data['referencePublication'].get('@id', '').startswith(self.prefixes['doi']):
            related_id = self.to_related_identifier(self.data['referencePublication']['@id'], 'isdocumentedby')
            self.add_to_array_field(zenodo_dict, 'related_identifiers', related_id)

        if self.data.get('codeRepository'):
            related_id = self.to_related_identifier(self.data['codeRepository'], 'issupplementto')
            self.add_to_array_field(zenodo_dict, 'related_identifiers', related_id)

        if 'author' in self.data:
            for author in self.data['author']:
                zenodo_creator = self.to_person_or_org(author)
                self.add_to_array_field(zenodo_dict, 'creators', zenodo_creator)

        if 'contributor' in self.data:
            for contributor in self.data['contributor']:
                if 'additionalType' in contributor and contributor['additionalType'] in self.ZENODO_CONTRIBUTORTYPES:
                    contributor_type = contributor['additionalType']
                else:
                    contributor_type = 'other'
                zenodo_contributor = self.to_person_or_org(contributor, contributor_type)
                self.add_to_array_field(zenodo_dict, 'contributors', zenodo_contributor)

        if 'description' in self.data:
            zenodo_dict['metadata']['description'] = self.data['description']

        if 'keywords' in self.data:
            zenodo_dict['metadata']['subjects'] = self.to_subjects(self.data['keywords'])

        if 'publisher' in self.data:
            zenodo_publisher = self.to_person_or_org(self.data['publisher'], 'distributor')
            self.add_to_array_field(zenodo_dict, 'contributors', zenodo_publisher)
            zenodo_dict['metadata']['publisher'] = \
                self.person_or_org_to_string(zenodo_publisher.get('person_or_org', {}))

        # Put it in "keywords" as no field seems to correspond
        if 'applicationCategory' in self.data:
            self.add_to_array_field(zenodo_dict,
                                    'subjects',
                                    {'subject': self.data['applicationCategory']}
                                    )

        if self.data.get('@type') == 'SoftwareSourceCode':
                zenodo_dict['metadata']['resource_type'] = {'id': 'software'}

        if 'license' in self.data:
            if not isinstance(self.data['license'], list):
                licenses = [self.data['license']]
            else:
                licenses = self.data['license']
            for license in licenses:
                self.add_to_array_field(zenodo_dict, 'rights', self.to_rights(license))

        if 'copyrightHolder' in self.data:
            for copyright_holder in self.data['copyrightHolder']:
                zenodo_rights_holder = self.to_person_or_org(copyright_holder, 'rightsholder')
                self.add_to_array_field(zenodo_dict, 'contributors', zenodo_rights_holder)

        if 'funding' in self.data:
            if not isinstance(self.data['funding'], list):
                fundings = [self.data['funding']]
            else:
                fundings = self.data['funding']
            for funding in fundings:
                zenodo_funding = self.to_funding(funding)
                if zenodo_funding:
                    self.add_to_array_field(zenodo_dict, 'funding', zenodo_funding)

        logger.debug('zenodo_dict = %s', json.dumps(zenodo_dict, indent=2))
        return zenodo_dict



import io
from os import path
from xml.dom.minidom import parseString
from xml.sax.saxutils import XMLGenerator

import pytest

from facile_rs.utils.metadata import CodemetaMetadata, DataciteMetadata

# Get current script location
SCRIPT_DIR = path.dirname(path.realpath(__file__))


@pytest.fixture
def create_metadata():
    codemeta = CodemetaMetadata()
    codemeta.fetch(path.join(SCRIPT_DIR, 'codemeta_test.json'))
    codemeta.compute_names()
    metadata = DataciteMetadata(codemeta.data)
    return codemeta, metadata


@pytest.fixture
def create_metadata_schemaorg():
    codemeta = CodemetaMetadata()
    codemeta.fetch(path.join(SCRIPT_DIR, 'codemeta_schemaorg_test.json'))
    codemeta.compute_names()
    metadata = DataciteMetadata(codemeta.data)
    return codemeta, metadata


def test_render_node(create_metadata):
    metadata = DataciteMetadata({})
    metadata.render_node('name', {'key': 'value', 'other': 'val'}, 'Test text')
    dom = parseString(metadata.stream.getvalue())
    assert dom.toxml() == '<?xml version="1.0" ?><name key="value" other="val">Test text</name>'


def test_render_funding_references_codemeta(create_metadata):
    """Test for CodeMeta formatted funding metadata
    """
    _, metadata = create_metadata
    expected_result = """<?xml version="1.0" ?><fundingReferences><fundingReference><funderName>Karlsruhe Institute"""\
        """ of Technology</funderName></fundingReference><fundingReference><funderName>TES_2024_TEST</funderName>"""\
        """</fundingReference></fundingReferences>"""
    metadata.render_funding_references()
    dom = parseString(metadata.stream.getvalue())
    assert dom.toxml() == expected_result


def test_render_funding_references_schemaorg(create_metadata_schemaorg):
    """Test for schema.org formatted funding metadata
    """
    _, metadata = create_metadata_schemaorg
    with open(path.join(SCRIPT_DIR,'datacite_schemaorg_funding_ref.xml')) as f:
        expected_result = f.read()
    metadata.render_funding_references()
    dom = parseString(metadata.stream.getvalue())
    print(dom.toprettyxml(indent='    '))
    assert dom.toprettyxml(indent='    ') == expected_result


def test_init(create_metadata):
    codemeta, metadata = create_metadata
    assert metadata.data == codemeta.data
    assert isinstance(metadata.stream, io.StringIO)
    assert isinstance(metadata.xml, XMLGenerator)


def test_conversion(create_metadata):
    _, metadata = create_metadata
    with open(path.join(SCRIPT_DIR, 'datacite_ref.xml')) as f:
        assert metadata.to_xml() == f.read()



import json
import sys
from datetime import date

from facile_rs.utils.cli import main

RELEASE_TAG = 'v0.1.0'
RELEASE_DATE = '2024-12-16'
MINIMAL_CODEMETA = """{
    "@context": "https://doi.org/10.5063/schema/codemeta-2.0",
    "@type": "SoftwareSourceCode",
    "name": "CodemetaR",
    "version": "0.0.1",
    "description": "Codemeta defines a 'JSON-LD' format for describing software metadata. This package provides utilities to generate, parse, and modify codemeta.jsonld files automatically for R packages.",
    "license": "https://spdx.org/licenses/GPL-3.0",
    "identifier": "http://dx.doi.org/10.5281/zenodo.XXXX"
}
"""  # noqa: E501


def assertions(initial_codemeta, modified_codemeta, release_tag, release_date):
    """
    Assert that the modified codemeta file has same content as the initial one,
    except for the version and dateModified fields.
    """
    assert modified_codemeta['version'] == release_tag
    assert modified_codemeta['dateModified'] == release_date
    for key in initial_codemeta:
        if key not in ['version', 'dateModified']:
            assert key in modified_codemeta
            assert initial_codemeta[key] == modified_codemeta[key]


def test_cli(monkeypatch, tmp_path):
    """
    Test the prepare_release script using command-line arguments and providing a date.
    """
    codemeta_location = tmp_path / 'codemeta.json'
    codemeta_location.write_text(MINIMAL_CODEMETA)
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'release',
                            'prepare',
                            '--codemeta-location', str(codemeta_location),
                            '--version', RELEASE_TAG,
                            '--date', RELEASE_DATE
                        ])
    main()
    modified_codemeta = json.loads(codemeta_location.read_text())
    initial_codemeta = json.loads(MINIMAL_CODEMETA)
    assertions(initial_codemeta, modified_codemeta, RELEASE_TAG, RELEASE_DATE)


def test_env_nodate(monkeypatch, tmp_path):
    """
    Test the prepare_release script using environment variables and not providing a date.
    """
    codemeta_location = tmp_path / 'codemeta.json'
    codemeta_location.write_text(MINIMAL_CODEMETA)
    monkeypatch.setenv('CODEMETA_LOCATION', str(codemeta_location))
    monkeypatch.setenv('VERSION', RELEASE_TAG)
    monkeypatch.delenv('DATE', raising=False)
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'release',
                            'prepare'
                        ])
    main()
    modified_codemeta = json.loads(codemeta_location.read_text())
    initial_codemeta = json.loads(MINIMAL_CODEMETA)
    assertions(initial_codemeta, modified_codemeta, RELEASE_TAG, date.today().strftime('%Y-%m-%d'))



import logging
import os
import shutil
import tempfile
from pathlib import Path

from dotenv import load_dotenv

logger = logging.getLogger(__file__)


class Settings:

    _shared_state = {}

    DEFAULTS = {
        'ASSETS_TOKEN_NAME': 'PRIVATE-TOKEN',
        'LOG_LEVEL': 'WARN',
        'DRY': False
    }

    def __init__(self):
        self.__dict__ = self._shared_state

    def __str__(self):
        return str(vars(self))

    def setup(self, parser, validate=[]):
        # setup env from .env file
        load_dotenv(Path().cwd() / '.env')

        # get the args from the parser
        args = parser.parse_args()

        # combine settings from args and os.environ
        args_dict = vars(args)
        for key, value in args_dict.items():
            attr = key.upper()

            if value not in [None, []]:
                attr_value = value
            elif os.environ.get(attr):
                if isinstance(value, list):
                    attr_value = os.environ.get(attr).split()
                else:
                    attr_value = os.environ.get(attr)
            else:
                if isinstance(value, list):
                    attr_value = self.DEFAULTS.get(attr, [])
                else:
                    attr_value = self.DEFAULTS.get(attr)

            setattr(self, attr, attr_value)

        # setup logs
        log_level = self.LOG_LEVEL.upper()
        log_file = Path(self.LOG_FILE).expanduser().as_posix() if self.LOG_FILE else None
        logging.basicConfig(level=log_level, filename=log_file,
                            format='[%(asctime)s] %(levelname)s %(name)s: %(message)s')

        # log settings
        logger.debug('settings = %s', self)

        # validate settings
        errors = []
        for key in validate:
            if getattr(self, key) is None:
                errors.append(key)
        if len(errors) == 1:
            parser.error(f'{errors[0]} is missing.')
        elif len(errors) >= 1:
            parser.error('{} are missing.'.format(', '.join(errors)))


settings = Settings()


def setup_assets_path(assets_path, remove_existing=False, exist_ok=False):
    path = Path(assets_path).expanduser()
    if remove_existing:
        shutil.rmtree(path)
    path.mkdir(parents=True, exist_ok=exist_ok)
    return path


def setup_tmp_assets_path():
    tmp_dir = tempfile.TemporaryDirectory(prefix='facile-rs', delete=False)
    return Path(tmp_dir.name), tmp_dir



import logging
from datetime import datetime

import yaml

logger = logging.getLogger(__file__)


class CffMetadata:

    """A class for creating CFF citation files from CodeMeta metadata"""

    doi_prefix = 'https://doi.org/'
    orcid_prefix = 'https://orcid.org/'
    ror_prefix = 'https://ror.org/'

    name_types = {
        'Person': 'Personal',
        'Organization': 'Organizational'
    }

    def __init__(self, data):
        """Initialization from a CodeMeta metadata dictionary.

        :param data: data attribute of a class:CodemetaMetadata instance
        :type data: dict
        """
        self.data = data
        logger.debug('data = %s', self.data)

    def to_yaml(self):
        """Convert metadata to CFF format.

        :return: Content of the CFF file
        :rtype: str
        """
        cff_json = {
            'cff-version': '1.2.0',
            'message': 'If you use this software, please cite the paper describing it as below. '
                       'Specific versions of the software can additionally be referenced using individual DOIs.',
            'type': 'software'
        }

        if 'name' in self.data:
            cff_json['title'] = self.data['name']

        if 'description' in self.data:
            cff_json['abstract'] = self.data['description']

        # Check if a DOI is provided
        id_keys = ['@id', 'identifier', 'id']
        for id_key in id_keys:
            if id_key in self.data and isinstance(self.data[id_key], str):
                if self.data[id_key].startswith(self.doi_prefix):
                    cff_json['doi'] = self.data[id_key].replace(self.doi_prefix, '')
                    break

        if 'sameAs' in self.data:
            cff_json['url'] = self.data['sameAs']

        if 'version' in self.data:
            cff_json['version'] = self.data['version']

        if 'dateModified' in self.data:
            cff_json['date-released'] = datetime.strptime(self.data['dateModified'], '%Y-%m-%d').date()

        if 'author' in self.data:
            cff_json['authors'] = []
            for author in self.data['author']:
                cff_author = {}
                is_person = False

                if 'givenName' in author:
                    cff_author['given-names'] = author['givenName']
                    is_person = True

                if 'familyName' in author:
                    cff_author['family-names'] = author['familyName']
                    is_person = True

                if not is_person and 'name' in author:
                    cff_author['name'] = author['name']

                if '@id' in author and author['@id'].startswith(self.orcid_prefix):
                    cff_author['orcid'] = author['@id']

                if cff_author:
                    cff_json['authors'].append(cff_author)

        if 'license' in self.data:
            if isinstance(self.data['license'], str):
                cff_json['license'] = self.data['license']
                # CodeMeta wants the license to be a URL or a creative work, CFF is only interested in the name
                if cff_json['license'].startswith("https://spdx.org/licenses/"):
                    cff_json['license'] = cff_json['license'].replace("https://spdx.org/licenses/", "")
            elif isinstance(self.data['license'], dict):
                self.data['license']['name']
                if 'name' in self.data['license']:
                    cff_json['license'] = self.data['license']['name']
                if 'url' in self.data['license']:
                    cff_json['license-url'] = self.data['license']['url']

        if 'codeRepository' in self.data:
            cff_json['repository-code'] = self.data['codeRepository']

        if 'referencePublication' in self.data:
            cff_json['preferred-citation'] = {}

            if '@type' in self.data['referencePublication']:
                if self.data['referencePublication']['@type'] == 'ScholarlyArticle':
                    cff_json['preferred-citation']['type'] = 'article'

            if '@id' in self.data['referencePublication'] and \
                    self.data['referencePublication']['@id'].startswith(self.doi_prefix):
                cff_json['preferred-citation']['doi'] = \
                    self.data['referencePublication']['@id'].replace(self.doi_prefix, '')

            if 'name' in self.data['referencePublication']:
                cff_json['preferred-citation']['title'] = self.data['referencePublication']['name']

            if 'isPartOf' in self.data['referencePublication']:
                if 'isPartOf' in self.data['referencePublication']['isPartOf']:
                    if 'name' in self.data['referencePublication']['isPartOf']['isPartOf']:
                        cff_json['preferred-citation']['journal'] = \
                            self.data['referencePublication']['isPartOf']['isPartOf']['name']

                if 'volumeNumber' in self.data['referencePublication']['isPartOf']:
                    cff_json['preferred-citation']['volume'] = \
                        int(self.data['referencePublication']['isPartOf']['volumeNumber'])

                if 'datePublished' in self.data['referencePublication']['isPartOf']:
                    cff_json['preferred-citation']['year'] = \
                        int(self.data['referencePublication']['isPartOf']['datePublished'])

            if 'pageStart' in self.data['referencePublication']:
                cff_json['preferred-citation']['pages'] = self.data['referencePublication']['pageStart']

            if 'pageEnd' in self.data['referencePublication']:
                cff_json['preferred-citation']['pages'] += "-" + self.data['referencePublication']['pageEnd']

            if 'author' in self.data['referencePublication']:
                cff_json['preferred-citation']['authors'] = []
                for author in self.data['referencePublication']['author']:
                    cff_citation_author = {}

                    if 'familyName' in author:
                        cff_citation_author['family-names'] = author['familyName']

                    if 'givenName' in author:
                        cff_citation_author['given-names'] = author['givenName']

                    if '@id' in author and author['@id'].startswith(self.orcid_prefix):
                        cff_citation_author['orcid'] = author['@id']

                    cff_json['preferred-citation']['authors'].append(cff_citation_author)

        # Case when identifier follows https://schema.org/identifier schema
        if 'identifier' in self.data:
            cff_identifiers = schema_org_identifiers_to_cff(self.data['identifier'])
            if cff_identifiers:
                cff_json['identifiers'] = schema_org_identifiers_to_cff(self.data['identifier'])

        # isBasedOn schema.org property
        # If the field contains entries with DOI identifiers, those are added
        # as references in the CFF file
        if 'isBasedOn' in self.data:
            references = []
            if isinstance(self.data['isBasedOn'], list):
                isBasedOn_list = self.data['isBasedOn']
            else:
                isBasedOn_list = [self.data['isBasedOn']]
            for isBasedOn_item in isBasedOn_list:
                if 'identifier' in isBasedOn_item:
                    cff_identifiers = schema_org_identifiers_to_cff(isBasedOn_item['identifier'])
                    if cff_identifiers:
                        references.append(
                            {
                                'identifiers': cff_identifiers
                            }
                        )
            if references:
                cff_json['references'] = references

        return yaml.dump(cff_json, allow_unicode=True, sort_keys=False, default_flow_style=False)


def schema_org_identifier_to_cff(identifier):
    """ Converts schema.org identifier to CFF identifier.
    Supports only DOI identifiers for now. Returns an empty dict if identifier is not a schema.org-compliant
    DOI identifier.

    :param identifier: schema.org compliant DOI identifier
    :type identifier: dict
    :return: CFF-formatted DOI identifier or empty dict if identifier was not compliant.
    :rtype: dict
    """
    cff_identifier = {}
    if 'propertyID' in identifier and identifier['propertyID'] == "DOI":
        if 'description' in identifier and isinstance(identifier['description'], str):
            cff_identifier['description'] = identifier['description']
        cff_identifier['type'] = 'doi'
        if 'value' in identifier:
            cff_identifier['value'] = identifier['value']
    return cff_identifier

def schema_org_identifiers_to_cff(identifiers):
    """ Converts schema.org identifier field to CFF identifiers.
    Supports only DOI identifiers for now.

    :param identifier: schema.org compliant DOI identifier field
    :type identifier: dict or list of dict
    :return: CFF-formatted DOI identifier list
    :rtype: list of dict
    """
    # Ensure to have a list of identifiers
    if isinstance(identifiers, dict):
        identifiers_list = [identifiers]
    else:
        identifiers_list = identifiers
    cff_identifiers = []
    for identifier in identifiers_list:
        cff_identifier = schema_org_identifier_to_cff(identifier)
        if cff_identifier:
            cff_identifiers.append(cff_identifier)
    return cff_identifiers



import ast
import json
import sys
from os import path

import pytest

from facile_rs.utils.cli import main

SCRIPT_DIR = path.dirname(path.realpath(__file__))
METADATA_DIR = path.join(path.dirname(SCRIPT_DIR), 'utils', 'metadata', 'tests')

CODEMETA_BASENAME = 'codemeta_test.json'
CODEMETA_LOCATION = path.join(METADATA_DIR, CODEMETA_BASENAME)
RADAR_URL = 'https://radar.kit.edu'
RADAR_USERNAME = 'testuser'
RADAR_PASSWORD = 'testpassword'
RADAR_CLIENT_ID = 'testclientid'
RADAR_CLIENT_SECRET = 'testclientsecret'
RADAR_WORKSPACE_ID = 'testworkspaceid'
RADAR_REDIRECT_URL = 'https://example.com'
RADAR_EMAIL = 'testuser@example.com'
RADAR_BACKLINK = 'https://example.com'
ASSETS = [CODEMETA_LOCATION, 'https://www.rfc-editor.org/rfc/rfc2606.txt']


def test_error_radar_asset_exists(monkeypatch, tmp_path, capsys):
    """
    Error is raised when a file with same name as the asset already exists in the radar path.
    """
    radar_path = tmp_path / 'radar_path'
    radar_path.mkdir()
    # Create a dummy file in the radar path to simulate an existing asset
    (radar_path / CODEMETA_BASENAME).touch()
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'radar',
                            'upload',
                            '--codemeta-location', CODEMETA_LOCATION,
                            '--radar-path', str(radar_path),
                            '--radar-url', RADAR_URL,
                            '--radar-username', RADAR_USERNAME,
                            '--radar-password', RADAR_PASSWORD,
                            '--radar-client-id', RADAR_CLIENT_ID,
                            '--radar-client-secret', RADAR_CLIENT_SECRET,
                            '--radar-workspace-id', RADAR_WORKSPACE_ID,
                            '--radar-redirect-url', RADAR_REDIRECT_URL,
                            '--radar-email', RADAR_EMAIL,
                            '--radar-backlink', RADAR_BACKLINK,
                            '--dry',
                            *ASSETS
                        ])
    with pytest.raises(SystemExit, match='^2$'):
        main()
    captured = capsys.readouterr().err
    assert 'already exists' in captured


def test_dry_cli(monkeypatch, tmp_path, capsys):
    """
    Test that the function runs with CLI options, that CodeMeta metadata is retrieved and that the
    Radar metadata is printed to stdout in dry mode.
    """
    radar_path = tmp_path / 'radar_path'
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'radar',
                            'upload',
                            '--codemeta-location', CODEMETA_LOCATION,
                            '--radar-path', str(radar_path),
                            '--radar-url', RADAR_URL,
                            '--radar-username', RADAR_USERNAME,
                            '--radar-password', RADAR_PASSWORD,
                            '--radar-client-id', RADAR_CLIENT_ID,
                            '--radar-client-secret', RADAR_CLIENT_SECRET,
                            '--radar-workspace-id', RADAR_WORKSPACE_ID,
                            '--radar-redirect-url', RADAR_REDIRECT_URL,
                            '--radar-email', RADAR_EMAIL,
                            '--radar-backlink', RADAR_BACKLINK,
                            '--dry'
                        ])
    main()
    captured = ast.literal_eval(capsys.readouterr().out)
    assert 'technicalMetadata' in captured
    assert 'descriptiveMetadata' in captured
    # Test if the title is correctly set to the name and version of the codemeta file
    with open(CODEMETA_LOCATION) as f:
        codemeta = json.load(f)
    codemeta_name = codemeta.get('name')
    codemeta_version = codemeta.get('version')
    assert captured['descriptiveMetadata'].get('title', '') == f'{codemeta_name} ({codemeta_version})'


def test_assets_env(monkeypatch, tmp_path, capsys):
    """
    Test the function runs with options set as environment variables and
    that assets are collected.
    """
    radar_path = tmp_path / 'radar_path'
    monkeypatch.setenv('RADAR_PATH', str(radar_path))
    monkeypatch.setenv('CODEMETA_LOCATION', CODEMETA_LOCATION)
    monkeypatch.setenv('RADAR_URL', RADAR_URL)
    monkeypatch.setenv('RADAR_USERNAME', RADAR_USERNAME)
    monkeypatch.setenv('RADAR_PASSWORD', RADAR_PASSWORD)
    monkeypatch.setenv('RADAR_CLIENT_ID', RADAR_CLIENT_ID)
    monkeypatch.setenv('RADAR_CLIENT_SECRET', RADAR_CLIENT_SECRET)
    monkeypatch.setenv('RADAR_WORKSPACE_ID', RADAR_WORKSPACE_ID)
    monkeypatch.setenv('RADAR_REDIRECT_URL', RADAR_REDIRECT_URL)
    monkeypatch.setenv('RADAR_EMAIL', RADAR_EMAIL)
    monkeypatch.setenv('RADAR_BACKLINK', RADAR_BACKLINK)
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'radar',
                            'upload',
                            '--dry',
                            *ASSETS
                        ])
    main()
    assert radar_path.is_dir()
    for asset in ASSETS:
        assert (radar_path / path.basename(asset)).is_file()



import json
import logging
import shutil
from pathlib import Path
from urllib.parse import urlparse

import requests
import yaml

from .exceptions import AssetExistsError

logger = logging.getLogger(__file__)


def fetch_files(locations, path, headers={}, overwrite=False):
    """
    Fetch files from local locations or from URLs and save them at the given path.

    :param locations: list of URL or paths to the files
    :type locations: list of str
    :param path: path where the files should be saved
    :type path: str
    """
    for location in locations:
        fetch_file(location, path, headers=headers, overwrite=overwrite)


def fetch_file(location, path, file_name=None, headers={}, overwrite=False):
    """
    Fetch one file from a local location or an URL and save them at the given path.
    If no file_name is provided it will be created from the location

    :param location: URL or path to the file
    :type locations: str or Path
    :param path: path where the file should be saved
    :type path: str or Path
    :param file_name: name of the file
    :type path: str
    """
    parsed_url = urlparse(location)
    target = path / (file_name or parsed_url.path.split('/')[-1])

    if target.exists() and not overwrite:
        raise AssetExistsError(location, target)

    logger.debug('location = %s, target = %s', location, target)
    if parsed_url.scheme:
        response = requests.get(location, headers=headers)
        response.raise_for_status()
        with open(target, 'wb') as f:
            f.write(response.content)

    else:
        shutil.copyfile(location, target)


def fetch_dict(location, headers={}):
    """Fetch data from a JSON or YAML file and return it as a dictionary.

    :param location: URL or path to the file (allowed extensions: .json, .yml, .yaml)
    :type location: str
    :return: Dictionary containing file data
    :rtype: dict
    """
    parsed_url = urlparse(location)
    if parsed_url.scheme:
        logger.debug('location = %s', location)

        response = requests.get(location, headers=headers)
        response.raise_for_status()

        content_type = response.headers.get('content-type')
        if content_type.startswith('application/json') or parsed_url.path.endswith('.json'):
            return json.loads(response.text)
        elif parsed_url.path.endswith('.yml') or parsed_url.path.endswith('.yaml'):
            return yaml.safe_load(response.text)
        else:
            raise RuntimeError(f'{location} is not a JSON or YAML file.')
    else:
        path = Path(location).expanduser()
        with open(Path(location).expanduser()) as f:
            if path.suffix == '.json':
                return json.load(f)
            elif path.suffix in ['.yml', '.yaml']:
                return yaml.safe_load(f.read())
            else:
                raise RuntimeError(f'{location} is not a JSON or YAML file.')


def fetch_json(location):
    """
    Fetch JSON data from the given location.

    :param location: URL to the JSON data
    :type location: str
    :return: JSON-encoded response
    """
    response = requests.get(location, headers={'Accept': 'application/json'})
    response.raise_for_status()
    return response.json()


def is_remote(location):
    """
    Check if a location is remote

    :param location: URL to the JSON data
    :return: bool
    """
    parsed_url = urlparse(location)
    return bool(parsed_url.scheme)



#!/usr/bin/env python3

"""Create subpages for published experiments to a Grav CMS repository.

Description
-----------

This script creates subpages for an experiments page from experiments from a collection on Zenodo or published
elsewhere with a DOI. It works together with https://git.opencarp.org/infrastructure/opencarp-grav-experiments-plugin
to create an "experiments marketplace".

The page need to be already existing in Grav and contain a ``pipeline`` field in their frontmatter. The script will
find all pages which match the provided PIPELINE and will add subpages to this page. The experiments themselves are
configured via the `experiments` object in the frontmatter of the page, e.g.:

```yaml
experiments:
    locations:
    - 'https://zenodo.org/api/communities/opencarp/records'
    gitlab_groups:
    - 'https://git.opencarp.org/api/v4/groups/153'
    dois:
    - 10.35097/763
    - 10.35097/606
    - 10.35097/580
    - 10.35097/1830
    - 10.35097/qec954fvka86gcjw
```

Please refer to https://opencarp.org/community/community-experiments for an example setup.

Usage
-----

.. argparse::
    :module: facile_rs.run_docstring_pipeline
    :func: create_parser
    :prog: run_docstring_pipeline.py

"""
import datetime
import logging
import re
import shutil

import frontmatter
from slugify import slugify

from .utils import cli
from .utils.grav import collect_pages
from .utils.http import fetch_dict

logger = logging.getLogger(__file__)

DATACITE_URL = 'https://api.datacite.org/dois/'
DATACITE_HEADERS = {
    'User-Agent': 'FACILE-RS/1.0 (https://git.opencarp.org/openCARP/FACILE-RS)'
}

def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('--grav-path', dest='GRAV_PATH', required=True,
                        help='Path to the grav repository directory.')
    parser.add_argument('--pipeline', dest='PIPELINE', required=True,
                        help='Name of the pipeline as specified in the GRAV metadata.')
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')
    return parser


def main(args):
    # loop over the found pages and write the content into the files
    for page_path, page, source in collect_pages(args.GRAV_PATH, args.PIPELINE):
        page_metadata = page.metadata.get('experiments', {})
        experiments_metadata = []

        # add pages from zenodo communities
        for location in page_metadata.get('locations', []):
            location_response = fetch_dict(location)

            if list(location_response.keys()) == ['hits', 'aggregations', 'links']:
                for hit in location_response['hits']['hits']:
                    metadata = hit['metadata']

                    experiments_metadata.append({
                        'doi': metadata.get('doi'),
                        'title': metadata.get('title'),
                        'description': metadata.get('description'),
                        'publication_date': metadata.get('publication_date'),
                        'publication_year': datetime.date.fromisoformat(metadata.get('publication_date')).year,
                        'resource_type': metadata.get('resource_type')['title'],
                        'creators': metadata.get('creators'),
                        'keywords': metadata.get('keywords'),
                        'version': metadata.get('version'),
                        'license': metadata.get('license')['id'].upper()
                    })
            else:
                raise RuntimeError('This pipeline can only process input from the invenio API.')

        # add pages from individual dois by querying datacite
        for doi in page_metadata.get('dois', []):
            datacite_response = fetch_dict(DATACITE_URL + doi, DATACITE_HEADERS)
            datacite_metadata = datacite_response.get('data', {}).get('attributes')

            experiments_metadata.append({
                'doi': doi,
                'title': get_datacite_title(datacite_metadata),
                'description': get_datacite_description(datacite_metadata),
                'publication_date': get_datacite_publication_date(datacite_metadata),
                'publication_year': datacite_metadata.get('publicationYear'),
                'resource_type': datacite_metadata['types'].get('resourceType'),
                'creators': [
                    {
                        'name': creator['name'],
                        'orcid': get_datacite_orcid(creator),
                    } for creator in datacite_metadata.get('creators', [])
                ],
                'keywords': [subject['subject'] for subject in datacite_metadata.get('subjects')],
                'version': datacite_metadata.get('version'),
                'license': get_datacite_license(datacite_metadata)
            })

        # collect project metadata from the gitlab group
        gitlab_metadata = {}
        for group_url in page_metadata.get('gitlab_groups', []):
            group = fetch_dict(group_url)

            for project in group['projects']:
                project_doi = False
                project_metadata = {
                    'gitlab_url': project['web_url'],
                    'gitlab_branch': project['default_branch'],
                    'gitlab_path': 'run.py'
                }

                for topic in project['topics']:
                    match_doi = re.match(r'doi:(.+)', topic)
                    if match_doi:
                        project_doi = match_doi.groups(1)[0]
                    match_opencarp_version = re.match(r'opencarp-version:(.+)', topic)
                    if match_opencarp_version:
                        project_metadata['opencarp_version'] = match_opencarp_version.groups(1)[0]

                if project_doi:
                    gitlab_metadata[project_doi] = project_metadata

        for metadata in experiments_metadata:
            metadata['slug'] = slugify(metadata['title'])
            metadata['template'] = 'experiment'
            metadata['doi_url'] = f"https://doi.org/{metadata['doi']}"
            metadata.update(gitlab_metadata.get(metadata['doi'], {}))

            md_path = page_path.parent / metadata['slug'] / 'default.md'

            # create directories in the grav tree
            md_path.parent.mkdir(parents=True, exist_ok=True)

            try:
                catalog_page = frontmatter.load(md_path)
                catalog_page.content = metadata['description']
                catalog_page.metadata.update(metadata)
            except FileNotFoundError:
                catalog_page = frontmatter.Post(metadata['description'], **metadata)

            # write the grav file
            logger.debug('writing to %s', md_path)
            md_path.write_text(frontmatter.dumps(catalog_page))

        for file_path in page_path.parent.iterdir():
            if file_path.is_dir() and file_path.stem not in [metadata['slug'] for metadata in experiments_metadata]:
                shutil.rmtree(file_path)


def get_datacite_title(metadata):
    for title in metadata.get('titles', []):
        return title['title']


def get_datacite_description(metadata):
    for description in metadata.get('descriptions', []):
        if description['descriptionType'] == 'Abstract':
            return description['description']


def get_datacite_publication_date(metadata):
    for date_type in ['Issued', 'Created']:
        for date in metadata.get('dates', []):
            if date['dateType'] == date_type:
                return date['date']


def get_datacite_orcid(creator):
    for name_identifier in creator.get('nameIdentifiers', []):
        if name_identifier['nameIdentifierScheme'] == 'ORCID':
            return name_identifier['nameIdentifier']


def get_datacite_license(metadata):
    for rights in metadata.get('rightsList', []):
        return rights['rights']



import json
import os

import pytest

from facile_rs.utils.http import fetch_files
from facile_rs.utils.zenodo import (
    create_zenodo_dataset,
    delete_zenodo_dataset,
    get_zenodo_dataset,
    prepare_zenodo_dataset,
    update_zenodo_dataset,
    upload_zenodo_assets,
)

SCRIPT_DIR = os.path.dirname(os.path.realpath(__file__))
METADATA_DIR = os.path.join(os.path.dirname(SCRIPT_DIR), 'metadata', 'tests')

CODEMETA_LOCATION = os.path.join(METADATA_DIR, 'codemeta_test.json')
MINIMAL_METADATA = {
    'metadata': {
        'language': 'eng',
        'title': 'FACILE-RS tests',
    }
}
ZENODO_METADATA_FILE = os.path.join(SCRIPT_DIR, 'zenodo_metadata.json')
ZENODO_TOKEN_PYTEST = os.getenv("ZENODO_TOKEN_PYTEST")
ZENODO_URL = 'https://sandbox.zenodo.org'


# Skip the tests if no Zenodo token is provided
@pytest.mark.skipif(
    os.getenv("ZENODO_TOKEN_PYTEST") is None,
    reason="ZENODO_TOKEN_PYTEST is not set."
)
def test_zenodo_utils(tmp_path):

    record_id = None

    try:

        # Test record creation
        record_id = create_zenodo_dataset(ZENODO_URL,
                                        ZENODO_TOKEN_PYTEST,
                                        MINIMAL_METADATA)
        assert record_id is not None

        # Test DOI reservation
        response_json = prepare_zenodo_dataset(ZENODO_URL, record_id, ZENODO_TOKEN_PYTEST)
        assert 'metadata' in response_json
        assert response_json['metadata']['title'] == MINIMAL_METADATA['metadata']['title']
        assert 'doi' in response_json['metadata']

        # Test record metadata update
        with open(ZENODO_METADATA_FILE) as f:
            metadata = json.load(f)
        update_zenodo_dataset(ZENODO_URL, record_id, ZENODO_TOKEN_PYTEST, metadata)
        record_json = get_zenodo_dataset(ZENODO_URL, record_id, ZENODO_TOKEN_PYTEST)
        assert 'metadata' in record_json
        assert record_json['metadata']['title'] == metadata['metadata']['title']

        # Test asset upload
        zenodo_path = tmp_path / 'zenodo-path'
        zenodo_path.mkdir()
        assets = [CODEMETA_LOCATION, 'https://www.rfc-editor.org/rfc/rfc2606.txt']
        fetch_files(assets, zenodo_path)
        upload_zenodo_assets(ZENODO_URL, record_id, ZENODO_TOKEN_PYTEST, assets, zenodo_path)
        record_json = get_zenodo_dataset(ZENODO_URL, record_id, ZENODO_TOKEN_PYTEST)
        assert 'files' in record_json
        assert len(record_json['files']) == 2
        for file in record_json['files']:
            assert file['key'] in (os.path.basename(asset) for asset in assets)

    finally:

        if record_id is not None:
            delete_zenodo_dataset(ZENODO_URL, record_id, ZENODO_TOKEN_PYTEST)



import hashlib


def get_sha256(file_path):
    """
    Get the SHA256 checksum of a file.
    """
    m = hashlib.sha256()
    m.update(file_path.read_text().encode())
    return m.hexdigest()


def get_sha512(file_path):
    """
    Get the SHA512 checksum of a file.
    """
    m = hashlib.sha512()
    m.update(file_path.read_text().encode())
    return m.hexdigest()



import logging
from urllib.parse import urlparse

import requests

logger = logging.getLogger(__file__)


def create_zenodo_dataset(zenodo_url, zenodo_token, zenodo_dict, previous_version=None):
    """
    Create a dataset in Zenodo, using the personal token provided.
    If a Zenodo ID is provided as `previous_version`, a new version of this Zenodo record will be created.

    :param zenodo_url: URL to the Zenodo repository
    :param zenodo_token: Zenodo personal token
    :type zenodo_token: str
    :param zenodo_dict: Zenodo metadata dictionary, as returned by ZenodoMetadata.as_dict()
    :type zenodo_dict: dict
    :param previous_version: ID of the record to create a new version of, if applicable
    :type previous_version: str or None
    :return: Zenodo dataset ID
    :rtype: str
    """
    url = zenodo_url + '/api/records'
    if previous_version:
        url += f'/{previous_version}/versions'
    headers = {
        "Content-Type": "application/json",
        "Authorization": "Bearer " + zenodo_token
    }
    try:
        response = requests.post(url,
                                headers=headers,
                                json=zenodo_dict)
        response.raise_for_status()
        logger.debug('response = %s', response.json())
        return response.json()['id']
    except requests.exceptions.HTTPError as e:
        logger.error(response.text)
        raise e


def prepare_zenodo_dataset(zenodo_url, dataset_id, zenodo_token):
    """"
    Prepare a dataset for review in ZENODO.

    :param zenodo_url: URL to the Zenodo repository
    :param dataset_id: Zenodo dataset ID
    :param zenodo_token: The Zenodo personal token to use for the upload
    :return: Zenodo response to the request
    """

    try:
        # Prereserve DOI
        headers = {"Authorization": "Bearer " + zenodo_token}
        reserve_doi_url = zenodo_url + f'/api/records/{dataset_id}/draft/pids/doi'
        response = requests.post(reserve_doi_url, headers=headers)
        response.raise_for_status()
        logger.debug('response = %s', response.json())
        # Get Zenodo dataset
        dataset_url = zenodo_url + f'/api/records/{dataset_id}/draft'
        response = requests.get(dataset_url, headers=headers)
        response.raise_for_status()
        logger.debug('response = %s', response.json())
        return response.json()
    except requests.exceptions.HTTPError as e:
        logger.error(response.text)
        raise e


def update_zenodo_dataset(zenodo_url, dataset_id, zenodo_token, zenodo_dict):
    """
    Update a dataset's metadata at the given Zenodo link.

    :param zenodo_url: URL to the Zenodo repository
    :param dataset_id: Zenodo dataset ID
    :param zenodo_token: Zenodo personal token
    :param zenodo_dict: Zenodo metadata dictionary, as returned by ZenodoMetadata.as_dict()
    :return: Zenodo dataset ID
    """
    url = zenodo_url + f'/api/records/{dataset_id}/draft'
    headers = {
        "Content-Type": "application/json",
        "Authorization": "Bearer " + zenodo_token
    }
    try:
        response = requests.put(url, headers=headers, json=zenodo_dict)
        response.raise_for_status()
        logger.debug('response = %s', response.json())
        return response.json()['id']
    except requests.exceptions.HTTPError as e:
        logger.error(response.text)
        raise e


def upload_zenodo_assets(zenodo_url, dataset_id, zenodo_token, assets, path):
    """
    Upload assets to a Zenodo dataset.

    :param zenodo_url: URL to the Zenodo repository
    :param dataset_id: Zenodo dataset ID
    :param zenodo_token: Zenodo personal token
    :param assets: locations of assets to upload
    :type assets: list
    :param path: location where the assets are collected before upload
    """
    headers = {
        "Authorization": "Bearer " + zenodo_token
    }

    for location in assets:
        parsed_url = urlparse(location)
        filename = parsed_url.path.split('/')[-1]
        target = path / filename

        # Start file upload
        try:
            response = requests.post(zenodo_url + f'/api/records/{dataset_id}/draft/files',
                                    headers=headers,
                                    json=[{'key': filename}])
            response.raise_for_status()
            logger.debug('response = %s', response.json())
        except requests.exceptions.HTTPError as e:
            logger.error(response.text)
            raise e

        # Upload file content
        with open(target, "rb") as fp:
            try:
                response = requests.put(zenodo_url + f'/api/records/{dataset_id}/draft/files/{filename}/content',
                                        headers=headers,
                                        data=fp)
                response.raise_for_status()
                logger.debug('response = %s', response.json())
            except requests.exceptions.HTTPError as e:
                logger.error(response.json())
                raise e

        # Complete file upload
        try:
            response = requests.post(zenodo_url + f'/api/records/{dataset_id}/draft/files/{filename}/commit',
                                    headers=headers)
            response.raise_for_status()
            logger.debug('response = %s', response.json())
        except requests.exceptions.HTTPError as e:
            logger.error(response.text)
            raise e


def delete_zenodo_dataset(zenodo_url, dataset_id, zenodo_token):
    """
    Delete a draft record from Zenodo.

    :param zenodo_url: URL to the Zenodo repository
    :param dataset_id: Zenodo record ID
    :param zenodo_token: Zenodo personal token
    """
    url = zenodo_url + f'/api/records/{dataset_id}/draft'
    headers = {
        "Authorization": "Bearer " + zenodo_token
    }
    try:
        response = requests.delete(url, headers=headers)
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        logger.error(response.text)
        raise e


def get_zenodo_dataset(zenodo_url, dataset_id, zenodo_token):
    """
    Get a record from Zenodo.

    :param zenodo_url: URL to the Zenodo repository
    :param dataset_id: Zenodo record ID
    :param zenodo_token: Zenodo personal token
    """
    url = zenodo_url + f'/api/records/{dataset_id}/draft'
    headers = {
        "Authorization": "Bearer " + zenodo_token
    }
    try:
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()
    except requests.exceptions.HTTPError as e:
        logger.error(response.text)
        raise e



#!/usr/bin/env python3

"""Create an archive in the RADAR service.

Description
-----------

This script creates an archive in the RADAR service and uploads the assets provided as positional arguments.
The metadata is created similar to create_datacite.

If the RADAR ID is already present in the CodeMeta file, the existing RADAR archive is updated instead.

Usage
-----

.. argparse::
    :module: facile_rs.create_radar
    :func: create_parser
    :prog: create_radar.py

"""
from rich import print

from .utils import cli, setup_assets_path, setup_tmp_assets_path
from .utils.exceptions import AssetExistsError, ParserError
from .utils.http import fetch_files
from .utils.mail import send_mail
from .utils.metadata import CodemetaMetadata, RadarMetadata
from .utils.radar import create_radar_dataset, fetch_radar_token, update_radar_dataset, upload_radar_assets


def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('ASSETS', nargs='*', default=[],
                        help='Assets to be added to the repository.')
    parser.add_argument('--codemeta-location', dest='CODEMETA_LOCATION', required=True,
                        help='Location of the main codemeta.json JSON file')
    parser.add_argument('--creators-locations', '--creators-location', dest='CREATORS_LOCATIONS',
                        action='append', default=[],
                        help='Locations of codemeta JSON files for additional creators')
    parser.add_argument('--contributors-locations', '--contributors-location', dest='CONTRIBUTORS_LOCATIONS',
                        action='append', default=[],
                        help='Locations of codemeta JSON files for additional contributors')
    parser.add_argument('--no-sort-authors', dest='SORT_AUTHORS', action='store_false', default=True,
                        help='Do not sort authors alphabetically, keep order in codemeta.json file')
    parser.add_argument('--radar-path', dest='RADAR_PATH',
                        help='Path to the local directory, where the assets are collected before upload. '
                             'Optional: if not provided, a temporary directory is used.')
    parser.add_argument('--radar-url', dest='RADAR_URL', required=True,
                        help='URL of the RADAR service.')
    parser.add_argument('--radar-username', dest='RADAR_USERNAME', required=True,
                        help='Username for the RADAR service.')
    parser.add_argument('--radar-password', dest='RADAR_PASSWORD', required=True,
                        help='Password for the RADAR service.')
    parser.add_argument('--radar-client-id', dest='RADAR_CLIENT_ID', required=True,
                        help='Client ID for the RADAR service.')
    parser.add_argument('--radar-client-secret', dest='RADAR_CLIENT_SECRET', required=True,
                        help='Client secret for the RADAR service.')
    parser.add_argument('--radar-workspace-id', dest='RADAR_WORKSPACE_ID', required=True,
                        help='Workspace ID for the RADAR service.')
    parser.add_argument('--radar-redirect-url', dest='RADAR_REDIRECT_URL', required=True,
                        help='Redirect URL for the OAuth workflow of the RADAR service.')
    parser.add_argument('--radar-email', dest='RADAR_EMAIL', required=True,
                        help='Email for the RADAR metadata.')
    parser.add_argument('--radar-backlink', dest='RADAR_BACKLINK', required=True,
                        help='Backlink for the RADAR metadata.')
    parser.add_argument('--smtp-server', dest='SMTP_SERVER',
                        help='SMTP server used to inform about new release. No mail sent if empty.')
    parser.add_argument('--notification-email', dest='NOTIFICATION_EMAIL',
                        help='Recipient address to inform about new release. No mail sent if empty.')
    parser.add_argument('--assets-token', dest='ASSETS_TOKEN',
                        help='Private token, to be used when fetching assets')
    parser.add_argument('--assets-token-name', dest='ASSETS_TOKEN_NAME', default='PRIVATE-TOKEN',
                        help='Name of the header field for the token [default: "PRIVATE-TOKEN"]')
    parser.add_argument('--dry', action='store_true', dest='DRY',
                        help='Perform a dry run, do not upload anything.')
    parser.add_argument('--overwrite', dest='OVERWRITE', action='store_true', env=False,
                        help='Overwrite existing local assets.')
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')
    return parser


def main(args):
    # setup the radar directory
    if args.RADAR_PATH is None:
        radar_path, tmp_dir = setup_tmp_assets_path()
    else:
        radar_path = setup_assets_path(args.RADAR_PATH, exist_ok=True)

    # collect assets
    try:
        fetch_files(args.ASSETS, radar_path, headers={
            args.ASSETS_TOKEN_NAME: args.ASSETS_TOKEN
        }, overwrite=args.OVERWRITE)
    except AssetExistsError as e:
        raise ParserError(f'Could not fetch {e.location}. File {e.file_path} already exists. '
                           'Use --overwrite to overwrite assets.') from e

    # prepare radar payload
    codemeta = CodemetaMetadata()
    codemeta.fetch(args.CODEMETA_LOCATION)
    codemeta.fetch_authors(args.CREATORS_LOCATIONS)
    codemeta.fetch_contributors(args.CONTRIBUTORS_LOCATIONS)
    codemeta.compute_names()
    codemeta.remove_doubles()
    if args.SORT_AUTHORS:
        codemeta.sort_persons()

    codemeta.data['name'] = '{name} ({version})'.format(**codemeta.data)  # override name/title to include version

    if not codemeta.data.get('copyrightHolder'):
        codemeta.data['copyrightHolder'] = [{
            'name': 'The authors'
        }]

    radar_metadata = RadarMetadata(codemeta.data, args.RADAR_EMAIL, args.RADAR_BACKLINK)
    radar_dict = radar_metadata.as_dict()

    if not args.DRY:
        # obtain oauth token
        headers = fetch_radar_token(args.RADAR_URL, args.RADAR_CLIENT_ID, args.RADAR_CLIENT_SECRET,
                                    args.RADAR_REDIRECT_URL, args.RADAR_USERNAME, args.RADAR_PASSWORD)

        # update or create radar dataset
        if radar_dict.get('id'):
            dataset_id = update_radar_dataset(args.RADAR_URL, radar_dict.get('id'), headers, radar_dict)
        else:
            dataset_id = create_radar_dataset(args.RADAR_URL, args.RADAR_WORKSPACE_ID, headers, radar_dict)

        # upload assets
        upload_radar_assets(args.RADAR_URL, dataset_id, headers, args.ASSETS, radar_path)
    else:
        print(radar_dict)

    if args.SMTP_SERVER and args.NOTIFICATION_EMAIL:
        radar_url = f'{args.RADAR_URL}/radar/de/workspace/{args.RADAR_WORKSPACE_ID}.{args.RADAR_CLIENT_ID}'

        send_mail(
            args.SMTP_SERVER, args.RADAR_EMAIL, args.NOTIFICATION_EMAIL,
            'New RADAR release ready to publish',
            'A new RADAR release has been uploaded by a CI pipeline.\n\n'
            f'Please visit {radar_url} to publish this release.'
        )

    try:
        tmp_dir.cleanup()
    except UnboundLocalError:
        pass


def main_deprecated():
    cli.main_deprecated(__name__)


if __name__ == "__main__":
    main_deprecated()



import ast
import json
import sys
from os import path

from facile_rs.utils.cli import main

SCRIPT_DIR = path.dirname(path.realpath(__file__))
METADATA_DIR = path.join(path.dirname(SCRIPT_DIR), 'utils', 'metadata', 'tests')

CODEMETA_LOCATION = path.join(METADATA_DIR, 'codemeta_test.json')
ZENODO_URL = 'https://sandbox.zenodo.org'
ZENODO_TOKEN = 'testtoken'


def test_dry_nocodemeta_cli(monkeypatch, capsys):
    """
    Test the function runs with CLI options and that the Zenodo metadata is printed in
    the standard output in dry mode.
    """
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'zenodo',
                            'prepare',
                            '--zenodo-url', ZENODO_URL,
                            '--zenodo-token', ZENODO_TOKEN,
                            '--dry'
                        ])
    main()
    captured = ast.literal_eval(capsys.readouterr().out)
    assert 'metadata' in captured
    assert captured['metadata'].get('title', '') == 'in preparation'


def test_dry_env(monkeypatch, capsys):
    """
    Test the function runs with options set as environment variables and
    that the CodeMeta metadata is handled correctly.
    """
    monkeypatch.setenv('CODEMETA_LOCATION', CODEMETA_LOCATION)
    monkeypatch.setenv('ZENODO_URL', ZENODO_URL)
    monkeypatch.setenv('ZENODO_TOKEN', ZENODO_TOKEN)
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'zenodo',
                            'prepare',
                            '--dry'
                        ])
    main()
    captured = ast.literal_eval(capsys.readouterr().out)
    assert 'metadata' in captured
    # Test if the title is correctly set to the name and version of the codemeta file
    with open(CODEMETA_LOCATION) as f:
        codemeta = json.load(f)
    codemeta_name = codemeta.get('name')
    codemeta_version = codemeta.get('version')
    assert captured['metadata'].get('title', '') == f'{codemeta_name} ({codemeta_version}, in preparation)'



import logging
import os
from pathlib import Path

import frontmatter

logger = logging.getLogger(__file__)


def collect_pages(grav_path, pipeline_name):
    """
    Collect pages in a GRAV repository which are associated with the given pipeline name.

    :param grav_path: path to the GRAV repository
    :type grav_path: string representing a path segment, or an object implementing the os.PathLike interface
    :param pipeline_name: name of the pipeline
    :type pipeline_name: str
    :return: list of pages as a tuple of (GRAV page location, GRAV page as frontmatter post, source location)
    :rtype: list
    """
    pages_path = Path(grav_path).expanduser() / 'pages'

    # walk the grav repo to find the files with `pipeline: carputils`
    pages = []
    for root, dirs, files in os.walk(pages_path):
        for file_name in files:
            file_path = Path(root) / file_name
            if file_path.suffix == '.md' and file_path.stem not in ['modular']:
                try:
                    page = frontmatter.load(file_path)
                    if page.get('pipeline') == pipeline_name:
                        logger.debug('file_path = %s', file_path)
                        pages.append((file_path, page, page.get('source')))

                except TypeError:
                    # if a file has issues with the metadata, just ignore it
                    pass

    return pages



#!/usr/bin/env python3

"""Copy the content of Markdown files to a Grav CMS repository.

Description
-----------

This script copies the content of markdown files in the PIPELINE_SOURCE to a Grav CMS repository given by GRAV_PATH.
The Grav repository is created by the Git-Sync Plugin.

The pages need to be already existing in Grav and contain a ``pipeline`` and a ``source`` field in their frontmatter.
The script will find all pages which match the provided PIPELINE and will overwrite content part of the page with the
markdown file given by source.
If source is ``codemeta.json``, the content will be added to the frontmatter entry ``codemeta`` rather than overwriting
the page content.
Twig templates digesting the metadata can be found in the file ``Twig_templates.md`` in this directory.
After running the script, the changes to the Grav CMS repository can be committed and pushed, and the Git-Sync Plugin
will update the public pages.
See openCARP citation info or code of conduct for examples.

Usage
-----

.. argparse::
    :module: facile_rs.run_markdown_pipeline
    :func: create_parser
    :prog: run_markdown_pipeline.py

"""
import json
import logging
from pathlib import Path

import frontmatter
import yaml

from .utils import cli
from .utils.grav import collect_pages

logger = logging.getLogger(__file__)


def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('--grav-path', dest='GRAV_PATH', required=True,
                        help='Path to the grav repository directory.')
    parser.add_argument('--pipeline', dest='PIPELINE', required=True,
                        help='Name of the pipeline as specified in the GRAV metadata.')
    parser.add_argument('--pipeline-source', dest='PIPELINE_SOURCE', required=True,
                        help='Path to the source directory for the pipeline.')
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')
    return parser


def main(args):
    # loop over the tagged pages and write the content into the files
    for page_path, page, source in collect_pages(args.GRAV_PATH, args.PIPELINE):
        source_path = Path(args.PIPELINE_SOURCE).expanduser() / source

        # read the source file
        if source_path.suffix in ['.json']:
            data = json.loads(source_path.read_text())
            if source_path.name == 'codemeta.json':
                page.metadata['codemeta'] = data
            else:
                page.metadata['items'] = data
        elif source_path.suffix in ['.yml', '.yaml']:
            page.metadata['items'] = yaml.safe_load(source_path.read_text())
        else:
            page.content = source_path.read_text()

        # write the page file
        logger.info('writing content to %s', page_path)
        page_path.write_text(frontmatter.dumps(page))


def main_deprecated():
    cli.main_deprecated(__name__)


if __name__ == "__main__":
    main_deprecated()



#!/usr/bin/env python3

"""Updates the CodeMeta file with the given ``version`` and ``date``.

Useful to automatically get the version from a git tag and inject it into the repository's metadata file.
The current date is used if no date is provided.

Usage
-----

.. argparse::
   :module: facile_rs.prepare_release
   :func: create_parser
   :prog: prepare_release.py

"""
from datetime import date
from pathlib import Path

from .utils import cli
from .utils.exceptions import ParserError
from .utils.http import is_remote
from .utils.metadata import CodemetaMetadata


def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('--codemeta-location', dest='CODEMETA_LOCATION', required=True,
                        help='Location of the main codemeta.json JSON file')
    parser.add_argument('--version', dest='VERSION', required=True,
                        help='Version of the resource')
    parser.add_argument('--date', dest='DATE',
                        help='Date for dateModified (format: \'%%Y-%%m-%%d\')')
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')
    return parser


def main(args):
    if is_remote(args.CODEMETA_LOCATION):
        raise ParserError(f'Location {args.CODEMETA_LOCATION} is not a local file.')

    codemeta = CodemetaMetadata()
    codemeta.fetch(args.CODEMETA_LOCATION)

    codemeta.data['version'] = args.VERSION
    codemeta.data['dateModified'] = args.DATE or date.today().strftime('%Y-%m-%d')

    Path(args.CODEMETA_LOCATION).expanduser().write_text(codemeta.to_json())


def main_deprecated():
    cli.main_deprecated(__name__)


if __name__ == "__main__":
    main_deprecated()



#!/usr/bin/env python
import os
import sys

if (
    (not os.path.exists("README.md"))
    and (not os.path.exists("README.txt"))
    and (not os.path.exists("README"))
):
    print(
        "Your repository does not contain a README file on the top level. Please consider adding one."
    )
    sys.exit(2)
else:
    print("README found.")



import json
from os import path

from facile_rs.utils.metadata import CodemetaMetadata

# Get current script location
SCRIPT_DIR = path.dirname(path.realpath(__file__))


def test_fetch():
    """Test fetching a JSON CodeMeta file
    """
    metadata = CodemetaMetadata()
    metadata.fetch(path.join(SCRIPT_DIR, 'codemeta_test.json'))
    with open(path.join(SCRIPT_DIR, 'codemeta_test.json')) as f:
        assert json.load(f) == metadata.data


def test_fetch_authors():
    """Test fetching authors from multiple files containing duplicates.
    """
    metadata = CodemetaMetadata()
    metadata.fetch(path.join(SCRIPT_DIR, 'codemeta_test.json'))
    metadata.fetch_authors([
        path.join(SCRIPT_DIR, 'codemeta_test.json'),
        path.join(SCRIPT_DIR, 'codemeta_authors_test.json')
        ])
    with open(path.join(SCRIPT_DIR, 'codemeta_authors_test.json')) as f:
        for author in json.load(f)['author']:
            assert author in metadata.data['author']


def test_fetch_contributors():
    """Test fetching contributors when initial dataset contains a unique contributor
    """
    metadata = CodemetaMetadata()
    metadata.fetch(path.join(SCRIPT_DIR, 'codemeta_test.json'))
    metadata.fetch_contributors([path.join(SCRIPT_DIR, 'codemeta_contributors_test.json')])
    with open(path.join(SCRIPT_DIR, 'codemeta_contributors_test.json')) as f:
        for contributor in json.load(f)['contributor']:
            print(contributor)
            print(metadata.data['contributor'])
            assert contributor in metadata.data['contributor']


def test_compute_names():
    metadata = CodemetaMetadata()
    metadata.data = {
        'author': [
            {'@type': 'Person', 'familyName': 'Author', 'givenName': 'One'}
        ],
        'contributor': [
            {'@type': 'Person', 'name': 'O Contributor', 'familyName': 'Contributor', 'givenName': 'One'},
            {'@id': 'http://orcid.org/9999-9999-9999-9999', '@type': 'Person',
             'email': 'another.contributor@example.com',
             'familyName': 'Contributor', 'givenName': 'Another'}
        ]
    }
    metadata.compute_names()
    # 'name' field doesn't exist
    assert metadata.data['author'][0]['name'] == 'One Author'
    assert metadata.data['contributor'][1]['name'] == 'Another Contributor'
    # 'name' field already exists and shouldn't be overwritten
    assert metadata.data['contributor'][0]['name'] == 'O Contributor'


def test_remove_doubles():
    metadata = CodemetaMetadata()
    metadata.data = {
        'author': [
            # Entries with same 'name' are considered duplicates
            {'@type': 'Person', 'name': 'One Author', 'email': 'oa@kit.edu'},
            {'@type': 'Person', 'name': 'One Author', 'email': 'oauthor@kit.edu'},
            {'@type': 'Person', 'familyName': 'Author', 'givenName': 'One', 'email': 'oa@kit.edu'}
        ],
        'contributor': [
            # Entries with same '@id' are considered duplicates
            {'@type': 'Person', 'familyName': 'Author', 'givenName': 'One', '@id': 'http://orcid.org/9999-9999-9999-9999'},
            {'@type': 'Person', 'familyName': 'Author', 'givenName': 'O', '@id': 'http://orcid.org/9999-9999-9999-9999'},
        ]
    }
    metadata.remove_doubles()
    expected_result = {
        'author': [
            {'@type': 'Person', 'name': 'One Author', 'email': 'oa@kit.edu'},
            {'@type': 'Person', 'familyName': 'Author', 'givenName': 'One', 'email': 'oa@kit.edu'}
        ],
        'contributor': [
            {'@type': 'Person', 'familyName': 'Author', 'givenName': 'One', '@id': 'http://orcid.org/9999-9999-9999-9999'}
        ]
    }
    assert metadata.data == expected_result


def test_sort_persons():
    metadata = CodemetaMetadata()
    for key in ['author', 'contributor']:
        metadata.data[key] = [
                {'@type': 'Person', 'name': 'Firstname Lastname'},
                {'@type': 'Person', 'familyName': 'Author', 'givenName': 'One', 'email': 'oa@kit.edu'},
                {'@type': 'Organization', 'name': 'First organization', 'additionalType': 'anotherType'},
                {'@type': 'Organization', 'name': 'Zyx organization'},
                {'@type': 'Organization', 'name': 'Another organization'}
            ]
    metadata.sort_persons()
    expected_result = {}
    for key in ['author', 'contributor']:
        expected_result[key] = [
                # Organizations first...
                {'@type': 'Organization', 'name': 'Another organization'},
                {'@type': 'Organization', 'name': 'Zyx organization'},
                {'@type': 'Person', 'familyName': 'Author', 'givenName': 'One', 'email': 'oa@kit.edu'},
                {'@type': 'Person', 'name': 'Firstname Lastname'},
                # ... Unless they have an additionalType
                {'@type': 'Organization', 'name': 'First organization', 'additionalType': 'anotherType'},
            ]
    assert expected_result == metadata.data


def test_update_identifier_new():
    # Test adding a new DOI and archive ID when no previous identifiers exist
    metadata = CodemetaMetadata()
    metadata.data = {
        'name': 'openCARP',
        'version': '0.0.1'
    }
    metadata.update_identifier(new_doi='10.1234/new.doi', new_archive_id='zenodo123', archive_type='Zenodo',
                               keep_previous_doi=False)
    expected_result = {
        'name': 'openCARP',
        'version': '0.0.1',
        '@id': 'https://doi.org/10.1234/new.doi',
        'identifier': [
            {
                '@type': 'PropertyValue',
                'propertyID': 'DOI',
                'value': '10.1234/new.doi',
                'description': 'The DOI for version 0.0.1 of this work'
            },
            {'@type': 'PropertyValue', 'propertyID': 'Zenodo', 'value': 'zenodo123'}
        ]
    }
    assert metadata.data == expected_result

def test_update_identifier_existing_overwrite():
    # Test adding a new DOI and archive ID when previous identifiers exist, without keeping the previous DOI
    metadata = CodemetaMetadata()
    metadata.data = {
        'name': 'openCARP',
        'version': '0.0.2',
        '@id': 'https://doi.org/10.1234/old.doi',
        'identifier': [
            {
                '@type': 'PropertyValue',
                'propertyID': 'DOI',
                'value': '10.1234/additional.doi',
                'description': 'The concept DOI of this work'
            },
            {
                '@type': 'PropertyValue',
                'propertyID': 'DOI',
                'value': '10.1234/old.doi',
                'description': 'The DOI for version 0.0.1 of this work'
            },
            {'@type': 'PropertyValue', 'propertyID': 'RADAR', 'value': 'radarold'}
        ]
    }
    metadata.update_identifier(new_doi='10.5678/new.doi', new_archive_id='radarnew', archive_type='RADAR',
                               keep_previous_doi=False)
    expected_result = {
        'name': 'openCARP',
        'version': '0.0.2',
        '@id': 'https://doi.org/10.5678/new.doi',
        'identifier': [
            {
                '@type': 'PropertyValue',
                'propertyID': 'DOI',
                'value': '10.1234/additional.doi',
                'description': 'The concept DOI of this work'
            },
            {
                '@type': 'PropertyValue',
                'propertyID': 'DOI',
                'value': '10.5678/new.doi',
                'description': 'The DOI for version 0.0.2 of this work'
            },
            {'@type': 'PropertyValue', 'propertyID': 'RADAR', 'value': 'radarnew'}
        ]
    }
    assert metadata.data == expected_result

def test_update_identifier_existing_keepdois():
    # Test adding a new DOI, keeping previous DOIs
    metadata = CodemetaMetadata()
    metadata.data = {
        'name': 'openCARP',
        'version': '1.0.0',
        '@id': 'https://doi.org/10.1234/old.doi',
        'identifier': [
            {
                '@type': 'PropertyValue',
                'propertyID': 'DOI',
                'value': '10.1234/additional.doi',
                'description': 'The concept DOI of this work'
            },
            {
                '@type': 'PropertyValue',
                'propertyID': 'DOI',
                'value': '10.1234/old.doi',
                'description': 'The DOI for version 0.0.1 of this work'
            },
            {'@type': 'PropertyValue', 'propertyID': 'RADAR', 'value': 'radarold'}
        ]
    }
    metadata.update_identifier(new_doi='10.5678/new.doi', keep_previous_doi=True)
    expected_result = {
        'name': 'openCARP',
        'version': '1.0.0',
        '@id': 'https://doi.org/10.5678/new.doi',
        'identifier': [
            {
                '@type': 'PropertyValue',
                'propertyID': 'DOI',
                'value': '10.1234/additional.doi',
                'description': 'The concept DOI of this work'
            },
            {
                '@type': 'PropertyValue',
                'propertyID': 'DOI',
                'value': '10.5678/new.doi',
                'description': 'The DOI for version 1.0.0 of this work'
            },
            {'@type': 'PropertyValue', 'propertyID': 'RADAR', 'value': 'radarold'}
        ],
        'isBasedOn': [
            {
                '@type': 'CreativeWork',
                'identifier': {
                    '@type': 'PropertyValue',
                    'propertyID': 'DOI',
                    'value': '10.1234/old.doi',
                    'description': 'The DOI for version 0.0.1 of this work'
                }
            }
        ]
    }
    assert metadata.data == expected_result



import io
import logging
from datetime import datetime
from xml.dom.minidom import parseString
from xml.sax.saxutils import XMLGenerator

logger = logging.getLogger(__file__)


class DataciteMetadata:

    """
    DataCite metadata generator.
    """

    doi_prefix = 'https://doi.org/'
    orcid_prefix = 'https://orcid.org/'
    ror_prefix = 'https://ror.org/'

    name_types = {
        'Person': 'Personal',
        'Organization': 'Organizational'
    }

    def __init__(self, data):
        """
        Initialize the metadata generator with CodeMeta metadata dictionary.

        :param data: CodeMeta metadata dictionary, typically the data attribute of a CodemetaMetadata object.
        """
        self.stream = io.StringIO()
        self.xml = XMLGenerator(self.stream, 'utf-8')

        self.data = data
        logger.debug('data = %s', self.data)

    def to_xml(self):
        """
        Generate the DataCite XML metadata.

        :return: DataCite metadata in XML format.
        """
        self.render_document()

        dom = parseString(self.stream.getvalue())
        xml = dom.toprettyxml(indent='    ')
        logger.debug('xml = %s', xml)
        return xml

    def render_node(self, tag, args={}, text=None):
        """
        Render a single XML node in self.xml .

        :param tag: XML tag name.
        :type tag: str
        :param args: XML tag attributes.
        :type args: dict
        :param text: XML tag text content.
        :type text: str
        """
        self.xml.startElement(tag, {k: v for k, v in args.items() if v is not None})
        if text:
            self.xml.characters(str(text))
        self.xml.endElement(tag)

    def render_document(self):
        """
        Render the whole DataCite XML document in self.xml .
        """
        self.xml.startDocument()
        self.xml.startElement('resource', {
            'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
            'xmlns': 'http://datacite.org/schema/kernel-4',
            'xsi:schemaLocation': 'http://datacite.org/schema/kernel-4 http://schema.datacite.org/meta/kernel-4.3/metadata.xsd'
        })

        if '@id' in self.data:
            if self.data['@id'].startswith(self.doi_prefix):
                self.render_node('identifier', {'identifierType': 'DOI'}, self.data['@id'].replace(self.doi_prefix, ''))
            else:
                self.render_node('identifier', {'identifierType': 'URL'}, self.data['@id'])

        if 'author' in self.data:
            self.xml.startElement('creators', {})
            for author in self.data['author']:
                if 'givenName' in author and 'familyName' in author:
                    name = '{} {}'.format(author['givenName'], author['familyName'])
                else:
                    name = author.get('name')

                if name is not None:
                    self.xml.startElement('creator', {})
                    self.render_node('creatorName', {
                        'nameType': self.name_types[author.get('@type', 'Person')]
                    }, name)

                    if 'givenName' in author:
                        self.render_node('givenName', {}, author['givenName'])

                    if 'familyName' in author:
                        self.render_node('familyName', {}, author['familyName'])

                    if '@id' in author:
                        if author['@id'].startswith(self.orcid_prefix):
                            self.render_node('nameIdentifier', {
                                'nameIdentifierScheme': 'ORCID',
                                'schemeURI': 'http://orcid.org'
                            }, author['@id'].replace(self.orcid_prefix, ''))

                    affiliations = author.get('affiliation', [])
                    # Case when there is only one affiliation that is not stored in a list
                    if isinstance(affiliations, dict):
                        affiliations = [affiliations]
                    for affiliation in affiliations:
                        affiliation_attrs = {}

                        if '@id' in affiliation:
                            if affiliation['@id'].startswith(self.ror_prefix):
                                affiliation_attrs = {
                                    'affiliationIdentifier': affiliation['@id'],
                                    'affiliationIdentifierScheme': 'ROR'
                                }

                        if 'name' in affiliation:
                            self.render_node('affiliation', affiliation_attrs, affiliation['name'])

                    self.xml.endElement('creator')
            self.xml.endElement('creators')

        if 'name' in self.data:
            self.xml.startElement('titles', {})
            self.render_node('title', {}, self.data['name'])
            if 'alternateName' in self.data:
                self.render_node('title', {
                    'titleType': 'AlternativeTitle'
                }, self.data['alternateName'])
            self.xml.endElement('titles')

        if 'publisher' in self.data and 'name' in self.data['publisher']:
            self.render_node('publisher', {}, self.data['publisher']['name'])

        if 'dateModified' in self.data:
            self.render_node('publicationYear', {}, datetime.strptime(self.data['dateModified'], '%Y-%m-%d').year)

        if 'keywords' in self.data:
            self.xml.startElement('subjects', {})
            for keyword in self.data['keywords']:
                if isinstance(keyword, str):
                    self.render_node('subject', {}, keyword)
                else:
                    if 'name' in keyword:
                        subject_args = {}
                        if 'inDefinedTermSet' in keyword:
                            subject_args['schemeURI'] = keyword['inDefinedTermSet']
                        if 'url' in keyword:
                            subject_args['valueURI'] = keyword['url']
                        self.render_node('subject', subject_args, keyword['name'])
            self.xml.endElement('subjects')

        contributors = self.data.get('contributor', [])
        # Case of a unique contributor
        if isinstance(contributors, dict):
            contributors = [contributors]
        copyright_holders = self.data.get('copyrightHolder', [])
        if isinstance(copyright_holders, dict):
            copyright_holders = [copyright_holders]
        contributors += copyright_holders
        if contributors:
            self.xml.startElement('contributors', {})
            for contributor in contributors:
                if 'givenName' in contributor and 'familyName' in contributor:
                    name = '{} {}'.format(contributor['givenName'], contributor['familyName'])
                else:
                    name = contributor.get('name')

                if name is not None:
                    if contributor in self.data.get('copyrightHolder', []):
                        contributor_type = 'RightsHolder'
                    elif 'additionalType' in contributor:
                        contributor_type = contributor['additionalType']
                    else:
                        contributor_type = None

                    self.xml.startElement('contributor', {
                        'contributorType': contributor_type
                    } if contributor_type is not None else {})
                    self.render_node('contributorName', {
                        'nameType': self.name_types[contributor.get('@type', 'Person')]
                    }, name)

                    if 'givenName' in contributor:
                        self.render_node('givenName', {}, contributor['givenName'])

                    if 'familyName' in contributor:
                        self.render_node('familyName', {}, contributor['familyName'])

                    if '@id' in contributor:
                        if contributor['@id'].startswith(self.orcid_prefix):
                            self.render_node('nameIdentifier', {
                                'nameIdentifierScheme': 'ORCID',
                                'schemeURI': 'http://orcid.org'
                            }, contributor['@id'].replace(self.orcid_prefix, ''))

                    if isinstance(contributor.get('affiliation', {}), dict):
                        contributor['affiliation'] = [contributor.get('affiliation', {})]
                    for affiliation in contributor.get('affiliation', []):
                        affiliation_attrs = {}

                        if '@id' in affiliation:
                            if affiliation['@id'].startswith(self.ror_prefix):
                                affiliation_attrs = {
                                    'affiliationIdentifier': affiliation['@id'],
                                    'affiliationIdentifierScheme': 'ROR'
                                }

                        if 'name' in affiliation:
                            self.render_node('affiliation', affiliation_attrs, affiliation['name'])

                    self.xml.endElement('contributor')
            self.xml.endElement('contributors')

        self.xml.startElement('dates', {})
        if 'dateCreated' in self.data:
            self.render_node('date', {
                'dateType': 'Created'
            }, self.data['dateCreated'])
        if 'dateModified' in self.data:
            self.render_node('date', {
                'dateType': 'Issued'
            }, self.data['dateModified'])
        self.xml.endElement('dates')

        self.render_node('language', {}, 'en-US')

        if 'applicationCategory' in self.data:
            self.render_node('resource', {
                'resourceType': 'Software' if self.data.get('@type') == 'SoftwareSourceCode' else None
            }, self.data['applicationCategory'])

        if 'sameAs' in self.data:
            self.xml.startElement('alternateIdentifiers', {})
            self.render_node('alternateIdentifier', {
                'alternateIdentifierType': 'URL'
            }, self.data['sameAs'])
            self.xml.endElement('alternateIdentifiers')

        if 'downloadUrl' in self.data:
            self.xml.startElement('alternateIdentifiers', {})
            self.render_node('alternateIdentifier', {
                'alternateIdentifierType': 'URL'
            }, self.data['downloadUrl'])
            self.xml.endElement('alternateIdentifiers')

        if any(key in self.data for key in ['referencePublication', 'codeRepository']):
            self.xml.startElement('relatedIdentifiers', {})
            if 'referencePublication' in self.data and \
                    self.data['referencePublication'].get('@id', '').startswith(self.doi_prefix):
                self.render_node('relatedIdentifier', {
                    'relatedIdentifierType': 'DOI',
                    'relationType': 'IsDocumentedBy'
                }, self.data['referencePublication']['@id'].replace(self.doi_prefix, ''))
            if 'codeRepository' in self.data:
                self.render_node('relatedIdentifier', {
                    'relatedIdentifierType': 'URL',
                    'relationType': 'IsSupplementTo'  # like zenodo does it
                }, self.data['codeRepository'])
            self.xml.endElement('relatedIdentifiers')

        if 'version' in self.data:
            self.render_node('version', {}, self.data['version'])

        if 'license' in self.data and 'name' in self.data['license']:
            self.xml.startElement('rightsList', {})
            self.render_node('rights', {
                'rightsURI': self.data['license'].get('url')
            }, self.data['license']['name'])
            self.xml.endElement('rightsList')

        if 'description' in self.data:
            self.xml.startElement('descriptions', {})
            self.render_node('description', {
                'descriptionType': 'Abstract'
            }, self.data['description'])
            self.xml.endElement('descriptions')

        self.render_funding_references()

        self.xml.endElement('resource')


    def render_funding_references(self):
        """DataCite XML rendering of funding references.
        See https://datacite-metadata-schema.readthedocs.io/en/4.5/properties/fundingreference/.

        According to the CodeMeta standard:
        - 'funder' is a (list of) Organization or Person (identical as schema.org).
        - 'funding' is a (list of) Text describing the funding grants.
        According to the schema.org standard:
        - 'funding' is a (list of) Grant (see https://schema.org/Grant)
        """

        if 'funder' not in self.data and 'funding' not in self.data:
            return

        self.xml.startElement('fundingReferences', {})

        # Create one funding reference per funder
        if 'funder' in self.data:
            funders = self.data['funder']
            # Turn unique element to list
            if isinstance(self.data['funder'], dict):
                funders = [funders]
            for funder in funders:
                self.xml.startElement('fundingReference', {})
                if 'name' in funder:
                    self.render_node('funderName', {}, funder['name'])
                if '@id' in funder and funder['@id'].startswith(self.ror_prefix):
                    self.render_node('funderIdentifier', {
                        'funderIdentifierType': 'ROR',
                        'schemeURI': 'https://ror.org'
                    }, funder['@id'])
                self.xml.endElement('fundingReference')

        # Create one funding reference per funding
        if 'funding' in self.data:
            fundings = self.data['funding']
            # Turn unique element to list
            if isinstance(self.data['funding'], (str, dict)):
                fundings = [fundings]
            for funding in fundings:
                self.xml.startElement('fundingReference', {})
                if isinstance(funding, str):
                    # Field funderName is mandatory
                    self.render_node('funderName', {}, funding)
                else:
                    if 'funder' in funding:
                        if 'name' in funding['funder']:
                            self.render_node('funderName', {}, funding['funder']['name'])
                        if '@id' in funding['funder'] and funding['funder']['@id'].startswith(self.ror_prefix):
                            self.render_node('funderIdentifier', {
                                'funderIdentifierType': 'ROR',
                                'schemeURI': 'https://ror.org'
                            }, funding['funder']['@id'])
                    if 'identifier' in funding:
                        self.render_node('awardNumber', {
                            'awardURI': funding.get('url')
                        }, funding['identifier'])
                    if 'name' in funding:
                        self.render_node('awardTitle', {}, funding['name'])
                self.xml.endElement('fundingReference')

        self.xml.endElement('fundingReferences')



#!/usr/bin/env python3

"""Create an archive in Zenodo.

Description
-----------

This script creates an archive in Zenodo and uploads the assets provided as positional arguments.
The metadata is created similar to create_datacite.

If the Zenodo ID is already present in the CodeMeta file, the existing Zenodo archive is updated instead.

Usage
-----

.. argparse::
    :module: facile_rs.create_zenodo
    :func: create_parser
    :prog: create_zenodo.py

"""

from rich import print

from .utils import cli, setup_assets_path, setup_tmp_assets_path
from .utils.exceptions import AssetExistsError, ParserError
from .utils.http import fetch_files
from .utils.mail import send_mail
from .utils.metadata import CodemetaMetadata, ZenodoMetadata
from .utils.zenodo import create_zenodo_dataset, update_zenodo_dataset, upload_zenodo_assets


def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('ASSETS', nargs='*', default=[],
                        help='Assets to be added to the repository.')
    parser.add_argument('--codemeta-location', dest='CODEMETA_LOCATION', required=True,
                        help='Location of the main codemeta.json JSON file')
    parser.add_argument('--creators-locations', '--creators-location', dest='CREATORS_LOCATIONS',
                        action='append', default=[],
                        help='Locations of codemeta JSON files for additional creators')
    parser.add_argument('--contributors-locations', '--contributors-location', dest='CONTRIBUTORS_LOCATIONS',
                        action='append', default=[],
                        help='Locations of codemeta JSON files for additional contributors')
    parser.add_argument('--no-sort-authors', dest='SORT_AUTHORS', action='store_false', default=True,
                        help='Do not sort authors alphabetically, keep order in codemeta.json file')
    parser.add_argument('--zenodo-path', dest='ZENODO_PATH',
                        help='Path to the local directory, where the assets are collected before upload. '
                             'Optional: if not provided, a temporary directory is used.')
    parser.add_argument('--zenodo-url', dest='ZENODO_URL', required=True,
                        help='URL of the Zenodo service. Test environment available at https://sandbox.zenodo.org')
    parser.add_argument('--zenodo-token', dest='ZENODO_TOKEN', required=True,
                        help='Zenodo personal token.')
    parser.add_argument('--smtp-server', dest='SMTP_SERVER',
                        help='SMTP server used to inform about new release. No mail sent if empty.')
    parser.add_argument('--notification-email', dest='NOTIFICATION_EMAIL',
                        help='Recipient address to inform about new release. No mail sent if empty.')
    parser.add_argument('--assets-token', dest='ASSETS_TOKEN',
                        help='Private token, to be used when fetching assets')
    parser.add_argument('--assets-token-name', dest='ASSETS_TOKEN_NAME', default='PRIVATE-TOKEN',
                        help='Name of the header field for the token [default: "PRIVATE-TOKEN"]')
    parser.add_argument('--dry', action='store_true', dest='DRY',
                        help='Perform a dry run, do not upload anything.')
    parser.add_argument('--overwrite', dest='OVERWRITE', action='store_true', env=False,
                        help='Overwrite existing local assets.')
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')
    return parser


def main(args):
    # setup the zenodo directory
    if args.ZENODO_PATH is None:
        zenodo_path, tmp_dir = setup_tmp_assets_path()
    else:
        zenodo_path = setup_assets_path(args.ZENODO_PATH, exist_ok=True)

    # collect assets
    try:
        fetch_files(args.ASSETS, zenodo_path, headers={
            args.ASSETS_TOKEN_NAME: args.ASSETS_TOKEN
        }, overwrite=args.OVERWRITE)
    except AssetExistsError as e:
        raise ParserError(f'Could not fetch {e.location}. File {e.file_path} already exists. '
                           'Use --overwrite to overwrite assets.') from e

    # prepare Zenodo payload
    codemeta = CodemetaMetadata()
    codemeta.fetch(args.CODEMETA_LOCATION)
    codemeta.fetch_authors(args.CREATORS_LOCATIONS)
    codemeta.fetch_contributors(args.CONTRIBUTORS_LOCATIONS)
    codemeta.compute_names()
    codemeta.remove_doubles()
    if args.SORT_AUTHORS:
        codemeta.sort_persons()

    # override name/title to include version
    codemeta.data['name'] = '{name} ({version})'.format(**codemeta.data)

    zenodo_metadata = ZenodoMetadata(codemeta.data)
    zenodo_dict = zenodo_metadata.as_dict()

    if not args.DRY:
        # update or create Zenodo dataset
        zenodo_id = None
        if 'identifier' in codemeta.data and isinstance(codemeta.data['identifier'], list):
            for identifier in codemeta.data['identifier']:
                if identifier.get('propertyID') == 'Zenodo':
                    zenodo_id = identifier['value']

        if zenodo_id:
            dataset_id = update_zenodo_dataset(args.ZENODO_URL, zenodo_id, args.ZENODO_TOKEN, zenodo_dict)
        else:
            dataset_id = create_zenodo_dataset(args.ZENODO_URL, args.ZENODO_TOKEN, zenodo_dict)

        # upload assets
        upload_zenodo_assets(args.ZENODO_URL, dataset_id, args.ZENODO_TOKEN, args.ASSETS, zenodo_path)

        if args.SMTP_SERVER and args.NOTIFICATION_EMAIL:
            zenodo_url = f'{args.ZENODO_URL}/uploads/{dataset_id}'

            send_mail(
                args.SMTP_SERVER, args.NOTIFICATION_EMAIL, args.NOTIFICATION_EMAIL,
                'New Zenodo release ready to publish',
                'A new Zenodo release has been uploaded by a CI pipeline.\n\n'
                f'Please visit {zenodo_url} to publish this release.'
            )

    try:
        tmp_dir.cleanup()
    except UnboundLocalError:
        pass

    else:
        print(zenodo_dict)


def main_deprecated():
    cli.main_deprecated(__name__)


if __name__ == "__main__":
    main_deprecated()



import sys
from os import path

import pytest

from facile_rs.utils.cli import main
from facile_rs.utils.http import fetch_dict

SCRIPT_DIR = path.dirname(path.realpath(__file__))
METADATA_DIR = path.join(path.dirname(SCRIPT_DIR), 'utils', 'metadata', 'tests')

BAG_INFO = path.join(SCRIPT_DIR, 'bag-info.yml')
CODEMETA_LOCATION = path.join(METADATA_DIR, 'codemeta_test.json')
CREATORS_LOCATIONS = path.join(METADATA_DIR, 'codemeta_authors_test.json')


def test_bag_path_exists(monkeypatch, tmp_path, capsys):
    """
    Error is raised when the bag path already exists
    """
    output_bag = tmp_path / 'bag'
    output_bag.mkdir()

    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'bag',
                            'create',
                            '--bag-path', str(output_bag)
                        ])
    # Check that SystemExit is raised with error code 2
    with pytest.raises(SystemExit, match='^2$'):
        main()
    # Check that the expected error is raised
    captured = capsys.readouterr().err
    assert 'already exists' in captured


def test_bag_cli(monkeypatch, tmp_path):
    """
    Test the bag creation using command line arguments:
        - bag-info.txt is created
        - The assets are copied to the bag/data directory
    """
    output_bag = tmp_path / 'bag'
    assets = [CODEMETA_LOCATION, CREATORS_LOCATIONS]
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'bag',
                            'create',
                            '--bag-path', str(output_bag),
                            *assets
                        ])
    main()
    assert output_bag.exists()
    assert (output_bag / 'bag-info.txt').exists()
    assert (output_bag / 'data' / 'codemeta_test.json').exists()
    assert (output_bag / 'data' / 'codemeta_authors_test.json').exists()


def test_bag_env(monkeypatch, tmp_path):
    """
    Test the bag creation using environment variables
        - bag-info.txt is created and populated with the expected elements
        - The assets are copied to the bag/data directory
    """
    output_bag = tmp_path / 'bag'
    assets = [CODEMETA_LOCATION, CREATORS_LOCATIONS]
    monkeypatch.setenv('BAG_PATH', str(output_bag))
    monkeypatch.setenv('BAG_INFO_LOCATIONS', str(BAG_INFO))
    monkeypatch.setenv('ASSETS', ' '.join(assets))
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'bag',
                            'create'
                        ])
    main()
    assert output_bag.exists()
    assert (output_bag / 'bag-info.txt').exists()
    assert (output_bag / 'data' / 'codemeta_test.json').exists()
    assert (output_bag / 'data' / 'codemeta_authors_test.json').exists()
    # Check that the bag-info file contains the expected elements
    out_bag_info = open(output_bag / 'bag-info.txt').read()
    bag_info = fetch_dict(BAG_INFO)
    for elem in bag_info.keys():
        assert elem in out_bag_info



import json
import logging
import re
from datetime import datetime, timezone

logger = logging.getLogger(__file__)


class RadarMetadata:

    doi_prefix = 'https://doi.org/'
    orcid_prefix = 'https://orcid.org/'
    ror_prefix = 'https://ror.org/'

    # map SPDX to Radar licenses
    radar_licenses = {
        "": "PUBLIC_DOMAIN_MARK_1_0",  # SPDX doesn't consider it a license https://github.com/spdx/license-list-XML/issues/988
        "PDDL-1.0": "PUBLIC_DOMAIN_DEDICATION_AND_LICENSE_PDDL",
        "ODC-By-1.0": "ATTRIBUTION_LICENSE_ODC_BY",
        "ODbL-1.0": "OPEN_DATABASE_LICENSE_ODC_O_DB_L",
        "Apache-2.0": "APACHE_LICENSE_2_0",
        "CDDL-1.0": "COMMON_DEVELOPMENT_AND_DISTRIBUTION_LICENSE_1_0",
        "EPL-1.0": "ECLIPSE_PUBLIC_LICENSE_1_0",
        "EPL-2.0": "ECLIPSE_PUBLIC_LICENSE_2_0",
        "GPL-3.0-only": "GNU_GENERAL_PUBLIC_LICENSE_V_3_0_ONLY",
        "LGPL-3.0": "GNU_LESSER_GENERAL_PUBLIC_LICENSE_V_3_0_ONLY",
        "BSD-2-Clause": "BSD_2_CLAUSE_SIMPLIFIED_LICENSE",
        "BSD-3-Clause": "BSD_3_CLAUSE_NEW_OR_REVISED_LICENSE",
        "MIT": "MIT_LICENSE"
    }

    def __init__(self, data, responsible_email, publication_backlink):
        """
        Initialize the RadarMetadata object from CodeMeta metadata.

        :param data: CodeMeta metadata, typically the data attribute of a CodemetaMetadata instance.
        :type data: dict
        :param responsible_email: email address of the contact person
        :type responsible_email: str
        :param publication_backlink: URL to the publication
        :type publication_backlink: str
        """
        self.data = data
        self.responsible_email = responsible_email
        self.publication_backlink = publication_backlink
        logger.debug('data = %s', self.data)

    def radar_value(self, string):
        """converts CamelCase to all caps and underscores, e.g. HostingInstitution -> HOSTING_INSTITUTION
        """
        return '_'.join([s.upper() for s in re.findall('([A-Z][a-z]+)', string)])

    def as_dict(self):
        """
        Prepare RADAR payload to be passed to the RADAR API.

        :return: RADAR metadata dictionary
        """
        archive_date = datetime.now(timezone.utc)

        radar_dict = {
            'technicalMetadata': {
                "retentionPeriod": 10,
                "archiveDate": int(archive_date.timestamp()),
                "responsibleEmail": self.responsible_email,
                "publicationBacklink": self.publication_backlink,
                "schema": {
                    "key": "RDDM",
                    "version": "9.2"
                }
            },
            'descriptiveMetadata': {
                'language': 'ENG',
                'resource': {}
            }
        }

        if 'identifier' in self.data and isinstance(self.data['identifier'], list):
            for identifier in self.data['identifier']:
                if identifier.get('propertyID') == 'RADAR':
                    radar_dict['id'] = identifier['value']

        if 'name' in self.data:
            radar_dict['descriptiveMetadata']['title'] = self.data['name']

        if 'dateModified' in self.data:
            production_year = datetime.strptime(self.data.get('dateModified'), '%Y-%m-%d')
            publish_date = datetime.strptime(self.data['dateModified'], '%Y-%m-%d')

            radar_dict['technicalMetadata']['publishDate'] = int(publish_date.timestamp())
            radar_dict['descriptiveMetadata']['productionYear'] = production_year.year
            radar_dict['descriptiveMetadata']['publicationYear'] = publish_date.year

        if 'sameAs' in self.data:
            radar_dict['descriptiveMetadata']['alternateIdentifiers'] = {
                'alternateIdentifier': []
            }
            radar_dict['descriptiveMetadata']['alternateIdentifiers']['alternateIdentifier'].append({
                'value': self.data['sameAs'],
                'alternateIdentifierType': 'URL'
            })

        if 'downloadUrl' in self.data:
            radar_dict['descriptiveMetadata']['alternateIdentifiers'] = {
                'alternateIdentifier': []
            }
            radar_dict['descriptiveMetadata']['alternateIdentifiers']['alternateIdentifier'].append({
                'value': self.data['downloadUrl'],
                'alternateIdentifierType': 'URL'
            })

        if any(key in self.data for key in ['referencePublication', 'codeRepository']):
            radar_dict['descriptiveMetadata']['relatedIdentifiers'] = {
                'relatedIdentifier': []
            }
            if 'referencePublication' in self.data and \
                    self.data['referencePublication'].get('@id', '').startswith(self.doi_prefix):
                radar_dict['descriptiveMetadata']['relatedIdentifiers']['relatedIdentifier'].append({
                    'value': self.data['referencePublication']['@id'].replace(self.doi_prefix, ''),
                    'relatedIdentifierType': 'DOI',
                    'relationType': self.radar_value('IsDocumentedBy')
                })
            if 'codeRepository' in self.data:
                radar_dict['descriptiveMetadata']['relatedIdentifiers']['relatedIdentifier'].append({
                    'value': self.data['codeRepository'],
                    'relatedIdentifierType': 'URL',
                    'relationType': self.radar_value('IsSupplementTo')  # like zenodo does it
                })

        if 'author' in self.data:
            radar_dict['descriptiveMetadata']['creators'] = {
                'creator': []
            }

            for author in self.data['author']:
                if 'name' in author:
                    radar_creator = {
                        'creatorName': author['name']
                    }

                    if 'givenName' in author:
                        radar_creator['givenName'] = author['givenName']

                    if 'familyName' in author:
                        radar_creator['familyName'] = author['familyName']

                    if '@id' in author:
                        if author['@id'].startswith(self.orcid_prefix):
                            radar_creator['nameIdentifier'] = [{
                                'value': author['@id'].replace(self.orcid_prefix, ''),
                                'schemeURI': 'http://orcid.org',
                                'nameIdentifierScheme': 'ORCID',
                            }]

                    author_affiliations = author.get('affiliation', [])
                    if not isinstance(author_affiliations, list):
                        author_affiliations = [author_affiliations]
                    for affiliation in author_affiliations:
                        if 'name' in affiliation:
                            radar_creator['creatorAffiliation'] = {
                                'value': affiliation['name']
                            }
                            if '@id' in affiliation and affiliation['@id'].startswith(self.ror_prefix):
                                radar_creator['creatorAffiliation'].update({
                                    'schemeURI': 'https://ror.org',
                                    'affiliationIdentifier': affiliation['@id'],
                                    'affiliationIdentifierScheme': 'ROR'
                                })

                    radar_dict['descriptiveMetadata']['creators']['creator'].append(radar_creator)

        if 'contributors' in self.data:
            radar_dict['descriptiveMetadata']['contributors'] = {
                'contributor': []
            }

            for contributor in self.data['contributors']:
                if 'name' in contributor:
                    radar_contributor = {
                        'contributorName': author['name'],
                        'contributorType': self.radar_value(contributor['additionalType'])
                    }

                    if contributor.get('@type') == 'Person':
                        radar_contributor['givenName'] = contributor['givenName']
                        radar_contributor['familyName'] = contributor['familyName']

                        if '@id' in contributor:
                            if contributor['@id'].startswith(self.orcid_prefix):
                                radar_contributor['nameIdentifier'] = [{
                                    'value': contributor['@id'].replace(self.orcid_prefix, ''),
                                    'schemeURI': 'http://orcid.org',
                                    'nameIdentifierScheme': 'ORCID',
                                }]

                        contributor_affiliations = contributor.get('affiliation', [])
                        if not isinstance(contributor_affiliations, list):
                            contributor_affiliations = [contributor_affiliations]
                        for affiliation in contributor_affiliations:
                            if 'name' in affiliation:
                                radar_creator['contributorAffiliation'] = {
                                    'value': affiliation['name']
                                }
                                if '@id' in affiliation and affiliation['@id'].startswith(self.ror_prefix):
                                    radar_creator['contributorAffiliation'].update({
                                        'schemeURI': 'https://ror.org',
                                        'affiliationIdentifier': affiliation['@id'],
                                        'affiliationIdentifierScheme': 'ROR'
                                    })

                radar_dict['descriptiveMetadata']['contributors']['contributor'].append(radar_contributor)

        if 'alternateName' in self.data:
            radar_dict['descriptiveMetadata']['additionalTitles'] = {
                'additionalTitle': []
            }
            radar_dict['descriptiveMetadata']['additionalTitles']['additionalTitle'].append({
                'value': self.data['alternateName'],
                'additionalTitleType': self.radar_value('AlternativeTitle')
            })

        if 'description' in self.data:
            radar_dict['descriptiveMetadata']['descriptions'] = {
                'description': []
            }
            radar_dict['descriptiveMetadata']['descriptions']['description'].append({
                'value': self.data['description'],
                'descriptionType': self.radar_value('Abstract')
            })

        if 'keywords' in self.data:
            keywords = []
            subjects = []
            for keyword in self.data['keywords']:
                if isinstance(keyword, str):
                    keywords.append(keyword)

                else:
                    if 'name' in keyword and \
                            keyword.get('@type') == 'DefinedTerm' and \
                            keyword.get('inDefinedTermSet', '').startswith('https://www.radar-service.eu/schemas/'):
                        subjects.append(keyword['name'])

            if keywords:
                radar_dict['descriptiveMetadata']['keywords'] = {
                    'keyword': []
                }
                for keyword in keywords:
                    radar_dict['descriptiveMetadata']['keywords']['keyword'].append({
                        'value': keyword
                    })

            if subjects:
                radar_dict['descriptiveMetadata']['subjectAreas'] = {
                    'subjectArea': []
                }
                for subject in subjects:
                    radar_dict['descriptiveMetadata']['subjectAreas']['subjectArea'].append({
                        'controlledSubjectAreaName': self.radar_value(subject)
                    })

        if 'publisher' in self.data and 'name' in self.data['publisher']:
            radar_dict['descriptiveMetadata']['publishers'] = {
                'publisher': []
            }
            radar_publisher = {
                'value': self.data['publisher']['name']
            }

            if '@id' in self.data['publisher'] and self.data['publisher']['@id'].startswith(self.ror_prefix):
                radar_publisher.update({
                    'schemeURI': 'https://ror.org',
                    'nameIdentifier': self.data['publisher']['@id'],
                    'nameIdentifierScheme': 'ROR'
                })

            radar_dict['descriptiveMetadata']['publishers']['publisher'].append(radar_publisher)

        if 'applicationCategory' in self.data:
            radar_dict['descriptiveMetadata']['resource'] = {
                'value': self.data['applicationCategory']
            }

        if self.data.get('@type') == 'SoftwareSourceCode':
                radar_dict['descriptiveMetadata']['resource']['resourceType'] = self.radar_value('Software')

        if 'license' in self.data:
            licenseName = ""
            if isinstance(self.data['license'], str):
                licenseName = self.data['license']
            elif isinstance(self.data['license'], dict) and 'name' in self.data['license']:
                licenseName = self.data['license']['name']

            # CodeMeta wants the license to be a URL or a creative work, RADAR is only interested in the name
            if licenseName.startswith("https://spdx.org/licenses/"):
                licenseName = licenseName.replace("https://spdx.org/licenses/", "")

            for spdx_name, radar_name in self.radar_licenses.items():
                if spdx_name:
                    licenseName = licenseName.replace(spdx_name, radar_name)

            if licenseName in list(self.radar_licenses.values()):
                radar_dict['descriptiveMetadata']['rights'] = {
                    'controlledRights': licenseName
                }
            else:
                radar_dict['descriptiveMetadata']['rights'] = {
                    'controlledRights': 'OTHER',
                    'additionalRights': licenseName
                }

        if 'copyrightHolder' in self.data:
            radar_dict['descriptiveMetadata']['rightsHolders'] = {
                'rightsHolder': []
            }
            for copyright_holder in self.data['copyrightHolder']:
                if 'name' in copyright_holder:
                    radar_rights_holder = {
                        'value': copyright_holder['name']
                    }

                    if '@id' in copyright_holder and copyright_holder['@id'].startswith(self.ror_prefix):
                        radar_rights_holder.update({
                            'schemeURI': 'https://ror.org',
                            'nameIdentifier': copyright_holder['@id'],
                            'nameIdentifierScheme': 'ROR'
                        })

                radar_dict['descriptiveMetadata']['rightsHolders']['rightsHolder'].append(radar_rights_holder)

        if 'funding' in self.data:
            radar_dict['descriptiveMetadata']['fundingReferences'] = {
                'fundingReference': []
            }
            for funding in self.data['funding']:
                radar_funding_reference = {}

                if 'funder' in funding:
                    if 'name' in funding['funder']:
                        radar_funding_reference['funderName'] = funding['funder']['name']

                    if '@id' in funding['funder'] and funding['funder']['@id'].startswith(self.ror_prefix):
                        radar_funding_reference['funderIdentifier'] = {
                            'value': funding['funder']['@id'],
                            'schemeURI': 'https://ror.org',
                            'type': 'ROR'
                        }

                if 'identifier' in funding:
                    radar_funding_reference['awardNumber'] = funding['identifier']
                if 'url' in funding:
                    radar_funding_reference['awardURI'] = funding['url']
                if 'name' in funding:
                    radar_funding_reference['awardTitle'] = funding['name']

                radar_dict['descriptiveMetadata']['fundingReferences']['fundingReference'].append(radar_funding_reference)

        logger.debug('radar_dict = %s', json.dumps(radar_dict, indent=2))
        return radar_dict



import sys
from os import path

from facile_rs.utils.cli import main

SCRIPT_DIR = path.dirname(path.realpath(__file__))
METADATA_DIR = path.join(path.dirname(SCRIPT_DIR), 'utils', 'metadata', 'tests')

CODEMETA_LOCATION = path.join(METADATA_DIR, 'codemeta_test.json')
CREATORS_LOCATIONS = path.join(METADATA_DIR, 'codemeta_authors_test.json')
CONTRIBUTORS_LOCATIONS = path.join(METADATA_DIR, 'codemeta_contributors_test.json')


def test_cli(monkeypatch, tmpdir):
    """
    Test the CFF file creation using command line arguments
    """
    output_cff = tmpdir.join('output.cff')
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'cff',
                            'create',
                            '--codemeta-location', CODEMETA_LOCATION,
                            '--creators-location', CREATORS_LOCATIONS,
                            '--contributors-location', CONTRIBUTORS_LOCATIONS,
                            '--cff-path', str(output_cff)
                        ])
    main()
    with open(path.join(SCRIPT_DIR, 'cff_ref.cff')) as cff_ref:
        assert output_cff.read() == cff_ref.read()


def test_env(monkeypatch, tmpdir):
    """
    Test the CFF file creation using environment variables
    """
    output_cff = tmpdir.join('output.cff')
    monkeypatch.setenv('CODEMETA_LOCATION', CODEMETA_LOCATION)
    monkeypatch.setenv('CREATORS_LOCATIONS', CREATORS_LOCATIONS)
    monkeypatch.setenv('CONTRIBUTORS_LOCATIONS', CONTRIBUTORS_LOCATIONS)
    monkeypatch.setenv('CFF_PATH', str(output_cff))
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'cff',
                            'create'
                        ])
    main()
    with open(path.join(SCRIPT_DIR, 'cff_ref.cff')) as cff_ref:
        assert output_cff.read() == cff_ref.read()

def test_stdout(monkeypatch, capsys):
    """
    Test the CFF output to stdout
    """
    # Increase the width of the terminal, needed since rich wraps prints
    monkeypatch.setenv('COLUMNS', '512')

    # Ensure no CFF_PATH environment variable is set
    monkeypatch.delenv('CFF_PATH', raising=False)
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'cff',
                            'create',
                            '--codemeta-location', CODEMETA_LOCATION,
                            '--creators-location', CREATORS_LOCATIONS,
                            '--contributors-location', CONTRIBUTORS_LOCATIONS,
                        ])
    main()
    captured = capsys.readouterr().out
    with open(path.join(SCRIPT_DIR, 'cff_ref.cff')) as cff_ref:
        # remove the last newline character added by the print function
        assert captured[:-1] == cff_ref.read()


def test_multiple_dois(monkeypatch, tmpdir):
    """
    Test handling of multiple DOIs in the CodeMeta file
    """
    codemeta_multiple_dois = path.join(METADATA_DIR, 'codemeta_multiple_dois.json')
    output_cff = tmpdir.join('output.cff')
    monkeypatch.setattr('sys.argv',
                        [
                            sys.argv[0],
                            'cff',
                            'create',
                            '--codemeta-location', codemeta_multiple_dois,
                            '--cff-path', str(output_cff)
                        ])
    main()
    with open(path.join(SCRIPT_DIR, 'cff_ref_multiple_dois.cff')) as cff_ref:
        assert output_cff.read() == cff_ref.read()



#!/usr/bin/env python3

"""Create a release in GitLab using the GitLab REST API.

Description
-----------

This script creates a release in GitLab using the GitLab REST API.
A tag for the release needs to be created beforehand and provided to the script.

Usage
-----

.. argparse::
   :module: facile_rs.create_release
   :func: create_parser
   :prog: create_release.py

"""
import logging

import requests
from rich import print

from .utils import cli

logger = logging.getLogger(__file__)


def create_parser(add_help=True):
    parser = cli.Parser(add_help=add_help)

    parser.add_argument('ASSETS', nargs='*', default=[],
                        help='Assets to be included in the release.')
    parser.add_argument('--release-tag', dest='RELEASE_TAG', required=True,
                        help='Tag for the release.')
    parser.add_argument('--release-description', dest='RELEASE_DESCRIPTION',
                        help='Description for the release.')
    parser.add_argument('--release-api-url', dest='RELEASE_API_URL', required=True,
                        help='API URL to create the release. Example: https://gitlab.com/api/v4/projects/123/releases')
    parser.add_argument('--private-token', dest='PRIVATE_TOKEN', required=True,
                        help='The PRIVATE_TOKEN to be used with the GitLab API.')
    parser.add_argument('--dry', action='store_true', dest='DRY',
                        help='Perform a dry run, do not perform the final request.')
    parser.add_argument('--log-level', dest='LOG_LEVEL', default='WARN',
                        help='Log level (ERROR, WARN, INFO, or DEBUG)')
    parser.add_argument('--log-file', dest='LOG_FILE',
                        help='Path to the log file')

    return parser


def main(args):
    if '/api/v4/projects/' not in args.RELEASE_API_URL or '/releases' not in args.RELEASE_API_URL:
        logger.warning('Warning: the RELEASE_API_URL seems incorrect. '
                       'It should be formed like: https://<gitlab_instance_url>/api/v4/projects/<project_id>/releases')

    assets = []
    for asset_location in args.ASSETS:
        assets.append({
            'name': asset_location.split('/')[-1],
            'url': asset_location
        })

    release_json = {
        'name': args.RELEASE_TAG,
        'tag_name': args.RELEASE_TAG
    }

    if args.RELEASE_DESCRIPTION:
        release_json['description'] = args.RELEASE_DESCRIPTION.strip()

    if assets:
        release_json['assets'] = {
            'links': assets
        }

    if args.DRY:
        print(release_json)
    else:
        logging.debug('release_json = %s', release_json)
        response = requests.post(args.RELEASE_API_URL, headers={
            'Content-Type': 'application/json',
            'Private-Token': args.PRIVATE_TOKEN
        }, json=release_json)
        response.raise_for_status()


def main_deprecated():
    cli.main_deprecated(__name__)


if __name__ == "__main__":
    main_deprecated()
